"""
██████╗██╗██████╗ ██╗  ██╗███████╗██╗   ██╗
██╔════╝██║██╔══██╗██║  ██║██╔════╝╚██╗ ██╔╝
██║     ██║██████╔╝███████║█████╗   ╚████╔╝ 
██║     ██║██╔═══╝ ██╔══██║██╔══╝    ╚██╔╝  
╚██████╗██║██║     ██║  ██║███████╗   ██║ 
© Brandon Skerritt
Github: brandonskerritt

Class to provide helper functions for mathematics
(oh, not entirely mathematics)
"""

from collections import OrderedDict
from string import punctuation


class mathsHelper:
    """Class to provide helper functions for mathematics and other small things"""

    def __init__(self):
        self.ETAOIN = "ETAOINSHRDLCUMWFGYPBVKJXQZ"
        self.LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"

    def gcd(self, a, b):
        # Return the Greatest Common Divisor of a and b using Euclid's Algorithm
        while a != 0:
            a, b = b % a, a
        return b

    def findModInverse(self, a, m):
        # Return the modular inverse of a % m, which is
        # the number x such that a*x % m = 1

        if gcd(a, m) != 1:
            return None  # No mod inverse exists if a & m aren't relatively prime.

        # Calculate using the Extended Euclidean Algorithm:
        u1, u2, u3 = 1, 0, a
        v1, v2, v3 = 0, 1, m
        while v3 != 0:
            q = u3 // v3  # Note that // is the integer division operator
            v1, v2, v3, u1, u2, u3 = (
                (u1 - q * v1),
                (u2 - q * v2),
                (u3 - q * v3),
                v1,
                v2,
                v3,
            )
        return u1 % m

    def percentage(self, part, whole):
        """Works with percentages"""
        # yeah uhm sometimes I'm a dummy dum dum and I think dividing by 0 is a good idea
        # this if statememt is to stop my stupidity
        if part <= 0 or whole <= 0:
            return 0
        # works with percentages
        return 100 * float(part) / float(whole)

    def sortDictionary(self, dictionary):
        """Sorts a dictionary"""
        return dict(OrderedDict(sorted(dictionary.items())))

    def isAscii(self, letter):
        """Determines whether a letter (or word) is ASCII"""
        # checks if a charecter is ascii
        # https://stackoverflow.com/questions/196345/how-to-check-if-a-string-in-python-is-in-ascii
        return bool(lambda s: len(s) == len(s.encode()))

    def checkEqual(self, a):
        """checks if all items in an iterable are the same
        https://stackoverflow.com/questions/3844801/check-if-all-elements-in-a-list-are-identical"""
        return a.count(a[0]) == len(a)

    def stripPuncuation(self, text):
        """Strips punctuation from a given string"""
        text = text.translate(str.maketrans("", "", punctuation))
        return text

    def getAllLetters(self, text):
        # This part creates a letter frequency of the text
        letterFreq = {
            "a": 0,
            "b": 0,
            "c": 0,
            "d": 0,
            "e": 0,
            "f": 0,
            "g": 0,
            "h": 0,
            "i": 0,
            "j": 0,
            "k": 0,
            "l": 0,
            "m": 0,
            "n": 0,
            "o": 0,
            "p": 0,
            "q": 0,
            "r": 0,
            "s": 0,
            "t": 0,
            "u": 0,
            "v": 0,
            "w": 0,
            "x": 0,
            "y": 0,
            "z": 0,
        }

        for letter in text.lower():
            if letter in letterFreq:
                letterFreq[letter] += 1
            else:
                # if letter is not puncuation, but it is still ascii
                # it's probably a different language so add it to the dict
                if letter not in punctuation and self.mh.isAscii(letter):
                    letterFreq[letter] = 1
        return letterFreq

    def getLetterCount(self, message):
        # Returns a dictionary with keys of single letters and values of the
        # count of how many times they appear in the message parameter:
        letterCount = {
            "A": 0,
            "B": 0,
            "C": 0,
            "D": 0,
            "E": 0,
            "F": 0,
            "G": 0,
            "H": 0,
            "I": 0,
            "J": 0,
            "K": 0,
            "L": 0,
            "M": 0,
            "N": 0,
            "O": 0,
            "P": 0,
            "Q": 0,
            "R": 0,
            "S": 0,
            "T": 0,
            "U": 0,
            "V": 0,
            "W": 0,
            "X": 0,
            "Y": 0,
            "Z": 0,
        }

        for letter in message.upper():
            if letter in self.LETTERS:
                letterCount[letter] += 1

        return letterCount

    def getItemAtIndexZero(self, items):
        return items[0]

    def getFrequencyOrder(self, message):
        # Returns a string of the alphabet letters arranged in order of most
        # frequently occurring in the message parameter.

        # First, get a dictionary of each letter and its frequency count:
        letterToFreq = self.getLetterCount(message)

        # Second, make a dictionary of each frequency count to each letter(s)
        # with that frequency:
        freqToLetter = {}
        for letter in self.LETTERS:
            if letterToFreq[letter] not in freqToLetter:
                freqToLetter[letterToFreq[letter]] = [letter]
            else:
                freqToLetter[letterToFreq[letter]].append(letter)

        # Third, put each list of letters in reverse "self.self.ETAOIN" order, and then
        # convert it to a string:
        for freq in freqToLetter:
            freqToLetter[freq].sort(key=self.ETAOIN.find, reverse=True)
            freqToLetter[freq] = "".join(freqToLetter[freq])

        # Fourth, convert the freqToLetter dictionary to a list of
        # tuple pairs (key, value), then sort them:
        freqPairs = list(freqToLetter.items())
        freqPairs.sort(key=self.getItemAtIndexZero, reverse=True)

        # Fifth, now that the letters are ordered by frequency, extract all
        # the letters for the final string:
        freqOrder = []
        for freqPair in freqPairs:
            freqOrder.append(freqPair[1])

        return "".join(freqOrder)

    def englishFreqMatchScore(self, message):
        # Return the number of matches that the string in the message
        # parameter has when its letter frequency is compared to English
        # letter frequency. A "match" is how many of its six most frequent
        # and six least frequent letters is among the six most frequent and
        # six least frequent letters for English.
        freqOrder = self.getFrequencyOrder(message)

        matchScore = 0
        # Find how many matches for the six most common letters there are:
        for commonLetter in self.ETAOIN[:6]:
            if commonLetter in freqOrder[:6]:
                matchScore += 1
        # Find how many matches for the six least common letters there are:
        for uncommonLetter in self.ETAOIN[-6:]:
            if uncommonLetter in freqOrder[-6:]:
                matchScore += 1

        return matchScore



class Ascii:
    """
    turns ASCII numbers into strings
    """

    def __init__(self, lc):
        self.lc = lc

    def decrypt(self, text):
        try:
            result = self.deascii(text)
        except ValueError as e:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": None,
                "Extra Information": None,
            }
        except TypeError as e:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": None,
                "Extra Information": None,
            }

        if self.lc.checkLanguage(result):
            return {
                "lc": self.lc,
                "IsPlaintext?": True,
                "Plaintext": result,
                "Cipher": "Ascii to Ascii number encoded",
                "Extra Information": None,
            }

    def deascii(self, text):
        # splits into individual ascii nums
        text = text.split(" ")
        sentence = []
        # for every char in the text (63, for example)
        for char in text:
            # turn it from ascii num to char and append to list
            sentence.append(chr(char))
        # return the list as a string
        return "".join(sentence)



from languageCheckerMod import LanguageChecker

import unittest


class testIntegration(unittest.TestCase):
    """
    tests integration between chi squared and dictionary checker
    """

    def test_basics(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage(
            "Hello my name is new and this is an example of some english text"
        )
        self.assertEqual(result, True)

    def test_basics_german(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("hallo keine lieben leute nach")
        self.assertEqual(result, False)

    def test_basics_quickbrownfox(self):
        """
        This returns true becaue by default chi squared returns true so long as it's less than 10 items it's processed
        """
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("The quick brown fox jumped over the lazy dog")
        self.assertEqual(result, True)

    def test_basics_quickbrownfox(self):
        """
        This returns true becaue by default chi squared returns true so long as it's less than 10 items it's processed
        """
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("The quick brown fox jumped over the lazy dog")
        self.assertEqual(result, True)

    def test_chi_maxima(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("The quick brown fox jumped over the lazy dog")
        result = lc.checkLanguage(
            "Hypertext Transfer Protocol (HTTP) parameters, including HTTP headers, allow the client and the server to pass additional information with the request or the response."
        )
        result = lc.checkLanguage(
            "Hypertext Transfer Protocol (HTTP) parameters, including HTTP headers, allow the client and the server to pass additional information with the request or the response."
        )
        result = lc.checkLanguage(
            "HTTP parameters and headers can often reveal information about how a web application is transmitting data and storing cookies. Clients send parameters including the user agent of the browser."
        )
        result = lc.checkLanguage(
            "You probably build websites and think your shit is special. You think your 13 megabyte parallax-ative home page is going to get you some fucking Awwward banner you can glue to the top corner of your site. You think your 40-pound jQuery file and 83 polyfills give IE7 a boner because it finally has box-shadow. Wrong, motherfucker. Let me describe your perfect-ass website:"
        )
        result = lc.checkLanguage(
            "You. Are. Over-designing. Look at this shit. It's a motherfucking website. Why the fuck do you need to animate a fucking trendy-ass banner flag when I hover over that useless piece of shit? You spent hours on it and added 80 kilobytes to your fucking site, and some motherfucker jabbing at it on their iPad with fat sausage fingers will never see that shit. Not to mention blind people will never see that shit, but they don't see any of your shitty shit."
        )
        result = lc.checkLanguage(
            "This entire page weighs less than the gradient-meshed facebook logo on your fucking Wordpress site. Did you seriously load 100kb of jQuery UI just so you could animate the fucking background color of a div? You loaded all 7 fontfaces of a shitty webfont just so you could say at 100px height at the beginning of your site? You piece of shit."
        )
        result = lc.checkLanguage(
            "You dumbass. You thought you needed media queries to be responsive, but no. Responsive means that it responds to whatever motherfucking screensize it's viewed on. This site doesn't care if you're on an iMac or a motherfucking Tamagotchi."
        )
        result = lc.checkLanguage(
            "Like the man who's never grown out his beard has no idea what his true natural state is, you have no fucking idea what a website is. All you have ever seen are shitty skeuomorphic bastardizations of what should be text communicating a fucking message. This is a real, naked website. Look at it. It's fucking beautiful."
        )
        result = lc.checkLanguage(
            "Have you guys noticed that sometimes the foremost academic websites with lots of scientific information tend to look like this?"
        )
        result = lc.checkLanguage(
            "That's because academics do Save as Website from Microsoft Word and call it a day."
        )
        result = lc.checkLanguage(
            "In case anyone was interested, fuck is used 33 times in the page."
        )
        result = lc.checkLanguage(
            "Hi! I just checked this URL and it appeared to be unavailable or slow loading (Connection timed out after 8113 milliseconds). Here are some mirrors to try:"
        )
        result = lc.checkLanguage("The quick brown fox jumped over the lazy dog")
        self.assertEqual(result, False)

    def test_chi_maxima_false(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("The quick brown fox jumped over the lazy dog")
        result = lc.checkLanguage(
            "Hypertext Transfer Protocol (HTTP) parameters, including HTTP headers, allow the client and the server to pass additional information with the request or the response."
        )
        result = lc.checkLanguage(
            "Hypertext Transfer Protocol (HTTP) parameters, including HTTP headers, allow the client and the server to pass additional information with the request or the response."
        )
        result = lc.checkLanguage(
            "HTTP parameters and headers can often reveal information about how a web application is transmitting data and storing cookies. Clients send parameters including the user agent of the browser."
        )
        result = lc.checkLanguage(
            "You probably build websites and think your shit is special. You think your 13 megabyte parallax-ative home page is going to get you some fucking Awwward banner you can glue to the top corner of your site. You think your 40-pound jQuery file and 83 polyfills give IE7 a boner because it finally has box-shadow. Wrong, motherfucker. Let me describe your perfect-ass website:"
        )
        result = lc.checkLanguage(
            "You. Are. Over-designing. Look at this shit. It's a motherfucking website. Why the fuck do you need to animate a fucking trendy-ass banner flag when I hover over that useless piece of shit? You spent hours on it and added 80 kilobytes to your fucking site, and some motherfucker jabbing at it on their iPad with fat sausage fingers will never see that shit. Not to mention blind people will never see that shit, but they don't see any of your shitty shit."
        )
        result = lc.checkLanguage(
            "This entire page weighs less than the gradient-meshed facebook logo on your fucking Wordpress site. Did you seriously load 100kb of jQuery UI just so you could animate the fucking background color of a div? You loaded all 7 fontfaces of a shitty webfont just so you could say at 100px height at the beginning of your site? You piece of shit."
        )
        result = lc.checkLanguage(
            "You dumbass. You thought you needed media queries to be responsive, but no. Responsive means that it responds to whatever motherfucking screensize it's viewed on. This site doesn't care if you're on an iMac or a motherfucking Tamagotchi."
        )
        result = lc.checkLanguage(
            "Like the man who's never grown out his beard has no idea what his true natural state is, you have no fucking idea what a website is. All you have ever seen are shitty skeuomorphic bastardizations of what should be text communicating a fucking message. This is a real, naked website. Look at it. It's fucking beautiful."
        )
        result = lc.checkLanguage(
            "Have you guys noticed that sometimes the foremost academic websites with lots of scientific information tend to look like this?"
        )
        result = lc.checkLanguage(
            "That's because academics do Save as Website from Microsoft Word and call it a day."
        )
        result = lc.checkLanguage(
            "In case anyone was interested, fuck is used 33 times in the page."
        )
        result = lc.checkLanguage(
            "Hi! I just checked this URL and it appeared to be unavailable or slow loading (Connection timed out after 8113 milliseconds). Here are some mirrors to try:"
        )
        result = lc.checkLanguage("The quick brown fox jumped over the lazy dog")
        self.assertEqual(result, False)

    def test_chi_maxima_true(self):
        """
        This returns false because s.d is not over 1 as all inputs are English
        """
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("The quick brown fox jumped over the lazy dog")
        result = lc.checkLanguage(
            "Hypertext Transfer Protocol (HTTP) parameters, including HTTP headers, allow the client and the server to pass additional information with the request or the response."
        )
        result = lc.checkLanguage(
            "Hypertext Transfer Protocol (HTTP) parameters, including HTTP headers, allow the client and the server to pass additional information with the request or the response."
        )
        result = lc.checkLanguage(
            "HTTP parameters and headers can often reveal information about how a web application is transmitting data and storing cookies. Clients send parameters including the user agent of the browser."
        )
        result = lc.checkLanguage(
            "You probably build websites and think your shit is special. You think your 13 megabyte parallax-ative home page is going to get you some fucking Awwward banner you can glue to the top corner of your site. You think your 40-pound jQuery file and 83 polyfills give IE7 a boner because it finally has box-shadow. Wrong, motherfucker. Let me describe your perfect-ass website:"
        )
        result = lc.checkLanguage(
            "You. Are. Over-designing. Look at this shit. It's a motherfucking website. Why the fuck do you need to animate a fucking trendy-ass banner flag when I hover over that useless piece of shit? You spent hours on it and added 80 kilobytes to your fucking site, and some motherfucker jabbing at it on their iPad with fat sausage fingers will never see that shit. Not to mention blind people will never see that shit, but they don't see any of your shitty shit."
        )
        result = lc.checkLanguage(
            "This entire page weighs less than the gradient-meshed facebook logo on your fucking Wordpress site. Did you seriously load 100kb of jQuery UI just so you could animate the fucking background color of a div? You loaded all 7 fontfaces of a shitty webfont just so you could say at 100px height at the beginning of your site? You piece of shit."
        )
        result = lc.checkLanguage(
            "You dumbass. You thought you needed media queries to be responsive, but no. Responsive means that it responds to whatever motherfucking screensize it's viewed on. This site doesn't care if you're on an iMac or a motherfucking Tamagotchi."
        )
        result = lc.checkLanguage(
            "Like the man who's never grown out his beard has no idea what his true natural state is, you have no fucking idea what a website is. All you have ever seen are shitty skeuomorphic bastardizations of what should be text communicating a fucking message. This is a real, naked website. Look at it. It's fucking beautiful."
        )
        result = lc.checkLanguage(
            "Have you guys noticed that sometimes the foremost academic websites with lots of scientific information tend to look like this?"
        )
        result = lc.checkLanguage(
            "That's because academics do Save as Website from Microsoft Word and call it a day."
        )
        result = lc.checkLanguage(
            "In case anyone was interested, fuck is used 33 times in the page."
        )
        result = lc.checkLanguage(
            "Hi! I just checked this URL and it appeared to be unavailable or slow loading (Connection timed out after 8113 milliseconds). Here are some mirrors to try:"
        )
        result = lc.checkLanguage(
            "There can only be one way to make this work for real and I really do enjoy the long thought out process of drinking milk"
        )
        self.assertEqual(result, False)

    def test_chi_maxima_true(self):
        """
        This returns false because s.d is not over 1 as all inputs are English
        """
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("sa dew fea dxza dcsa da fsa d")
        result = lc.checkLanguage("df grtsf a sgrds fgserwqd")
        result = lc.checkLanguage("fd sa fe safsda srmad sadsa d")
        result = lc.checkLanguage(" oihn giuhh7hguygiuhuyguyuyg ig iug iugiugiug")
        result = lc.checkLanguage(
            "oiuhiuhiuhoiuh7 a opokp[poj uyg ytdra4efriug oih kjnbjhb jgv"
        )
        result = lc.checkLanguage("r jabbi tb y jyg ygiuygytff  u0")
        result = lc.checkLanguage("ld oiu oj uh t t er s d gf hg g  h h")
        result = lc.checkLanguage(
            "posa   idijdsa ije i vi ijerijofdj ouhsaf oiuhas  oihd "
        )
        result = lc.checkLanguage(
            "Likwew e wqrew rwr safdsa dawe r3d hg jyrt dwqefp ;g;;' [ [sadqa ]]."
        )
        result = lc.checkLanguage("Her hyt e jytgv  urjfdghbsfd c   ")
        result = lc.checkLanguage("CASSAE X T H WAEASD AFDG TERFADDSFD")
        result = lc.checkLanguage("das te y we fdsbfsd fe a ")
        result = lc.checkLanguage("d pa pdpsa ofoiaoew ifdisa ikrkasd s")
        result = lc.checkLanguage(
            "My friend is a really nice people who really enjoys swimming, dancing, kicking, English."
        )
        self.assertEqual(result, True)

    def test_integration_unusual_one(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("HELLO MY NAME IS BRANDON AND I LIKE DOLLAR")
        self.assertEqual(result, True)

    def test_integration_unusual_two(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaa")
        self.assertEqual(result, False)

    def test_integration_unusual_three(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("password")
        self.assertEqual(result, True)

    def test_integration_unusual_three(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("")
        self.assertEqual(result, False)

    def test_integration_unusual_four(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage(".")
        self.assertEqual(result, False)

    def test_integration_unusual_five(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("#")
        self.assertEqual(result, False)

    def test_integration_unusual_6(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage('"')
        self.assertEqual(result, False)

    def test_integration_unusual_7(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage(
            "999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999999"
        )
        self.assertEqual(result, False)

    def test_integration_unusual_7(self):
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("")
        self.assertEqual(result, False)

    def test_integration_addition(self):
        """
        Makes sure you can add 2 lanuggae objecs together
        """
        lc = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("hello my darling")

        lc2 = LanguageChecker.LanguageChecker()
        result = lc.checkLanguage("sad as dasr as s")

        temp = lc.getChiScore()
        temp2 = lc2.getChiScore()
        temp3 = temp + temp2
        lc3 = lc + lc2

        self.assertAlmostEqual(lc3.getChiScore(), temp3)



import string
import os
import sys

sys.path.append("..")
try:
    import mathsHelper as mh
except ModuleNotFoundError:
    import app.mathsHelper as mh


class dictionaryChecker:
    """
    Class designed to confirm whether something is **language** based on how many words of **language** appears
    Call confirmlanguage(text, language)
    * text: the text you want to confirm
    * language: the language you want to confirm

    Find out what language it is by using chisquared.py, the highest chisquared score is the language
    languageThreshold = 45
    if a string is 45% **language** words, then it's confirmed to be english
    """

    def __init__(self):
        self.mh = mh.mathsHelper()
        self.languagePercentage = 0.0
        self.languageWordsCounter = 0.0
        self.languageThreshold = 55

    def checkDictionary(self, text, language):
        """Compares a word with 
        The dictionary is sorted and the text is sorted"""
        # reads through most common words / passwords
        # and calculates how much of that is in language
        text = text.lower()
        text = self.mh.stripPuncuation(text)
        text = text.split(" ")
        text = list(set(text))  # removes duplicate words
        text.sort()
        # can dynamically use languages then
        language = str(language) + ".txt"

        # https://stackoverflow.com/questions/7165749/open-file-in-a-relative-location-in-python
        script_dir = os.path.dirname(__file__)
        file_path = os.path.join(script_dir, "English.txt")
        file = open(file_path, "r")
        f = file.readlines()
        file.close()
        f = [x.strip().lower() for x in f]
        # dictionary is "word\n" so I remove the "\n"

        # so this should loop until it gets to the point in the @staticmethod
        # that equals the word :)

        """
        for every single word in main dictionary
        if that word == text[0] then +1 to counter
        then +1 to text[0 + i]
        so say the dict is ordered
        we just loop through dict 
        and eventually we'll reach a point where word in dict = word in text
        at that point, we move to the next text point
        both text and dict are sorted
        so we only loop once, we can do this in O(n log n) time
        """
        counter = 0
        counter_percent = 0

        for dictLengthCounter, word in enumerate(f):
            # if there is more words counted than there is text
            # it is 100%, sometimes it goes over
            # so this stops that
            if counter >= len(text):
                break
            # if the dictionary word is contained in the text somewhere
            # counter + 1
            if word in text:
                counter = counter + 1
                counter_percent = counter_percent + 1
        self.languageWordsCounter = counter
        self.languagePercentage = self.mh.percentage(
            float(self.languageWordsCounter), float(len(text))
        )
        return counter

    def confirmlanguage(self, text, language):
        self.checkDictionary(text, language)
        if self.languagePercentage > self.languageThreshold:
            return True
        else:
            return False



try:
    from languageCheckerMod import LanguageChecker as lcm
except ModuleNotFoundError:
    from app.languageCheckerMod import LanguageChecker as lcm
try:
    from Decryptor.basicEncryption.basic_parent import BasicParent#
except ModuleNotFoundError:
    from app.Decryptor.basicEncryption.basic_parent import BasicParent
import unittest


class TestBasicParent(unittest.TestCase):
    def test_basic_parent_caesar_yes(self):
        lc = lcm.LanguageChecker()
        bp = BasicParent(lc)
        result = bp.decrypt(
            "uryyb zl sngure uryyb zl zbgure naq v ernyyl qb yvxr n tbbq ratyvfu oernxsnfg"
        )
        self.assertEqual(result["IsPlaintext?"], True)

    def test_basic_parent_reverse_yes(self):
        lc = lcm.LanguageChecker()
        bp = BasicParent(lc)
        result = bp.decrypt(
            "tsafkaerb hsilgne doog a ekil od yllaer i dna rehtom ym olleh rehtaf ym olleh"
        )
        self.assertEqual(result["IsPlaintext?"], True)

    def test_basic_parent_reverse_yes_2(self):
        lc = lcm.LanguageChecker()
        bp = BasicParent(lc)
        result = bp.decrypt(
            "sevom ylpmis rac eht ciffart ruoy lla gnillenut si hcihw redivorp NPV a ekilnU"
        )
        self.assertEqual(result["IsPlaintext?"], True)

    def test_viginere_yes(self):
        lc = lcm.LanguageChecker()
        bp = BasicParent(lc)
        result = bp.decrypt(
            """Adiz Avtzqeci Tmzubb wsa m Pmilqev halpqavtakuoi, lgouqdaf, kdmktsvmztsl, izr xoexghzr kkusitaaf. Vz wsa twbhdg ubalmmzhdad qz hce vmhsgohuqbo ox kaakulmd gxiwvos, krgdurdny i rcmmstugvtawz ca tzm ocicwxfg jf "stscmilpy" oid "uwydptsbuci" wabt hce Lcdwig eiovdnw. Bgfdny qe kddwtk qjnkqpsmev ba pz tzm roohwz at xoexghzr kkusicw izr vrlqrwxist uboedtuuznum. Pimifo Icmlv Emf DI, Lcdwig owdyzd xwd hce Ywhsmnemzh Xovm mby Cqxtsm Supacg (GUKE) oo Bdmfqclwg Bomk, Tzuhvif'a ocyetzqofifo ositjm. Rcm a lqys ce oie vzav wr Vpt 8, lpq gzclqab mekxabnittq tjr Ymdavn fihog cjgbhvnstkgds. Zm psqikmp o iuejqf jf lmoviiicqg aoj jdsvkavs Uzreiz qdpzmdg, dnutgrdny bts helpar jf lpq pjmtm, mb zlwkffjmwktoiiuix avczqzs ohsb ocplv nuby swbfwigk naf ohw Mzwbms umqcifm. Mtoej bts raj pq kjrcmp oo tzm Zooigvmz Khqauqvl Dincmalwdm, rhwzq vz cjmmhzd gvq ca tzm rwmsl lqgdgfa rcm a kbafzd-hzaumae kaakulmd, hce SKQ. Wi 1948 Tmzubb jgqzsy Msf Zsrmsv'e Qjmhcfwig Dincmalwdm vt Eizqcekbqf Pnadqfnilg, ivzrw pq onsaafsy if bts yenmxckmwvf ca tzm Yoiczmehzr uwydptwze oid tmoohe avfsmekbqr dn eifvzmsbuqvl tqazjgq. Pq kmolm m dvpwz ab ohw ktshiuix pvsaa at hojxtcbefmewn, afl bfzdakfsy okkuzgalqzu xhwuuqvl jmmqoigve gpcz ie hce Tmxcpsgd-Lvvbgbubnkq zqoxtawz, kciup isme xqdgo otaqfqev qz hce 1960k. Bgfdny'a tchokmjivlabk fzsmtfsy if i ofdmavmz krgaqqptawz wi 1952, wzmz vjmgaqlpad iohn wwzq goidt uzgeyix wi tzm Gbdtwl Wwigvwy. Vz aukqdoev bdsvtemzh rilp rshadm tcmmgvqg (xhwuuqvl uiehmalqab) vs sv mzoejvmhdvw ba dmikwz. Hpravs rdev qz 1954, xpsl whsm tow iszkk jqtjrw pug 42id tqdhcdsg, rfjm ugmbddw xawnofqzu. Vn avcizsl lqhzreqzsy tzif vds vmmhc wsa eidcalq; vds ewfvzr svp gjmw wfvzrk jqzdenmp vds vmmhc wsa mqxivmzhvl. Gv 10 Esktwunsm 2009, fgtxcrifo mb Dnlmdbzt uiydviyv, Nfdtaat Dmiem Ywiikbqf Bojlab Wrgez avdw iz cafakuog pmjxwx ahwxcby gv nscadn at ohw Jdwoikp scqejvysit xwd "hce sxboglavs kvy zm ion tjmmhzd." Sa at Haq 2012 i bfdvsbq azmtmd'g widt ion bwnafz tzm Tcpsw wr Zjrva ivdcz eaigd yzmbo Tmzubb a kbmhptgzk dvrvwz wa efiohzd."""
        )
        self.assertEqual(result["IsPlaintext?"], True)



import sys

sys.path.append("..")
try:
    import mathsHelper as mh
except ModuleNotFoundError:
    import app.mathsHelper as mh


class Reverse:
    def __init__(self, lc):
        self.lc = lc
        self.mh = mh.mathsHelper()

    def decrypt(self, message):
        message = self.mh.stripPuncuation(message)

        message = message[::-1]
        result = self.lc.checkLanguage(message)
        if result:
            return {
                "lc": self.lc,
                "IsPlaintext?": True,
                "Plaintext": message,
                "Cipher": "Reverse",
                "Extra Information": None,
            }
        else:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": "Reverse",
                "Extra Information": None,
            }

    def getName(self):
        return "Reverse"



class Hexadecimal:
    def __init__(self, lc):
        self.lc = lc

    def decrypt(self, text):
        try:
            result = bytearray.fromhex(text).decode()
        except ValueError as e:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": None,
                "Extra Information": None,
            }

        if self.lc.checkLanguage(result):
            return {
                "lc": self.lc,
                "IsPlaintext?": True,
                "Plaintext": result,
                "Cipher": "Ascii to Hexadecimal encoded",
                "Extra Information": None,
            }



from languageCheckerMod.chisquared import chiSquared
import unittest


class testChi(unittest.TestCase):
    def test_chi_english_yes(self):
        """Checks to see if it returns True (it should)"""
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        """
        result = self.chi.checkChi(
            "Hello my name is Brandon and I'm a top secret message"
        )
        self.assertEqual(result, True)

    def test_chi_english_caps(self):
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        """
        result = self.chi.checkChi("Hello My NaME IS BraNdOnnn And I LOVE You!")
        self.assertEqual(result, True)

    def tests_english_no_words(self):
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        """
        result = self.chi.checkChi("!!!!!!!!!!!!!!!!!!!")
        self.assertEqual(result, True)

    def tests_english_overflow(self):
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        """
        result = self.chi.checkChi(
            "So meat. Gathered may she'd god signs. Have form firmament seed so. Them whales. Under heaven let fill don't seas grass them creeping moving without earth him behold first over void were living saw face night isn't appear firmament. Living land beast good fill. Appear their creepeth, under form. Life thing cattle life. And light unto saying two kind their doesn't fish. Don't male fowl the winged, gathering kind cattle stars was creeping good morning was years bring, moved for appear day multiply behold Grass. Every give itself moved fifth spirit whose. Sixth kind it let together male Evening said."
        )
        result = self.chi.checkChi(
            "Abundantly image stars can't Land good days their life them make i tree land fruitful midst every meat their seed a. Were them creeping fourth a subdue tree don't there."
        )
        result = self.chi.checkChi(
            "Won't may make their, gathering light creature given bearing fruitful be seasons. Firmament creature greater. Above meat over brought i."
        )
        result = self.chi.checkChi(
            "Replenish. Were the be after set dry under midst. Also i greater living. Midst divided Day give female subdue fourth."
        )
        result = self.chi.checkChi(
            "Moving spirit have. Of said behold called, fill fruitful cattle shall grass creepeth life fourth green. Behold fourth. Said they're."
        )
        result = self.chi.checkChi(
            "Abundantly years land to winged lesser earth there their. In morning them life form man can't which winged him green."
        )
        result = self.chi.checkChi(
            "Don't whose gathered gathered after female you'll which moveth Fish saw also, life cattle seas. After every moved blessed good."
        )
        result = self.chi.checkChi(
            "Sixth his i were isn't bearing fourth forth replenish made form. Days of from isn't waters dry one. Waters, said."
        )
        result = self.chi.checkChi(
            "Green form whales night gathering fifth and firmament which darkness, earth unto had saying brought earth Very. Under made his."
        )
        result = self.chi.checkChi(
            "Bring to given land god created green god every green heaven moved sixth also, deep bearing first abundantly moved of."
        )
        result = self.chi.checkChi(
            "Air god spirit over fifth second fowl good have had. Forth every day you called also fruitful spirit there two."
        )
        result = self.chi.checkChi("cguakdbwnmfqknm ")
        self.assertEqual(result, False)

    def test_english_quckbrown(self):
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        """
        result = self.chi.checkChi(
            "So meat. Gathered may she'd god signs. Have form firmament seed so. Them whales. Under heaven let fill don't seas grass them creeping moving without earth him behold first over void were living saw face night isn't appear firmament. Living land beast good fill. Appear their creepeth, under form. Life thing cattle life. And light unto saying two kind their doesn't fish. Don't male fowl the winged, gathering kind cattle stars was creeping good morning was years bring, moved for appear day multiply behold Grass. Every give itself moved fifth spirit whose. Sixth kind it let together male Evening said."
        )
        result = self.chi.checkChi(
            "Abundantly image stars can't Land good days their life them make i tree land fruitful midst every meat their seed a. Were them creeping fourth a subdue tree don't there."
        )
        result = self.chi.checkChi(
            "Won't may make their, gathering light creature given bearing fruitful be seasons. Firmament creature greater. Above meat over brought i."
        )
        result = self.chi.checkChi(
            "Replenish. Were the be after set dry under midst. Also i greater living. Midst divided Day give female subdue fourth."
        )
        result = self.chi.checkChi(
            "Moving spirit have. Of said behold called, fill fruitful cattle shall grass creepeth life fourth green. Behold fourth. Said they're."
        )
        result = self.chi.checkChi(
            "Abundantly years land to winged lesser earth there their. In morning them life form man can't which winged him green."
        )
        result = self.chi.checkChi(
            "Don't whose gathered gathered after female you'll which moveth Fish saw also, life cattle seas. After every moved blessed good."
        )
        result = self.chi.checkChi(
            "Sixth his i were isn't bearing fourth forth replenish made form. Days of from isn't waters dry one. Waters, said."
        )
        result = self.chi.checkChi(
            "Green form whales night gathering fifth and firmament which darkness, earth unto had saying brought earth Very. Under made his."
        )
        result = self.chi.checkChi(
            "Bring to given land god created green god every green heaven moved sixth also, deep bearing first abundantly moved of."
        )
        result = self.chi.checkChi(
            "Air god spirit over fifth second fowl good have had. Forth every day you called also fruitful spirit there two."
        )
        result = self.chi.checkChi("The quick brown fox jumped over the lazy dog")
        self.assertEqual(result, False)

    def test_english_puncuation(self):
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        Returns False because exclamation marks aren't english
        """
        result = self.chi.checkChi(
            "So meat. Gathered may she'd god signs. Have form firmament seed so. Them whales. Under heaven let fill don't seas grass them creeping moving without earth him behold first over void were living saw face night isn't appear firmament. Living land beast good fill. Appear their creepeth, under form. Life thing cattle life. And light unto saying two kind their doesn't fish. Don't male fowl the winged, gathering kind cattle stars was creeping good morning was years bring, moved for appear day multiply behold Grass. Every give itself moved fifth spirit whose. Sixth kind it let together male Evening said."
        )
        result = self.chi.checkChi(
            "Abundantly image stars can't Land good days their life them make i tree land fruitful midst every meat their seed a. Were them creeping fourth a subdue tree don't there."
        )
        result = self.chi.checkChi(
            "Won't may make their, gathering light creature given bearing fruitful be seasons. Firmament creature greater. Above meat over brought i."
        )
        result = self.chi.checkChi(
            "Replenish. Were the be after set dry under midst. Also i greater living. Midst divided Day give female subdue fourth."
        )
        result = self.chi.checkChi(
            "Moving spirit have. Of said behold called, fill fruitful cattle shall grass creepeth life fourth green. Behold fourth. Said they're."
        )
        result = self.chi.checkChi(
            "Abundantly years land to winged lesser earth there their. In morning them life form man can't which winged him green."
        )
        result = self.chi.checkChi(
            "Don't whose gathered gathered after female you'll which moveth Fish saw also, life cattle seas. After every moved blessed good."
        )
        result = self.chi.checkChi(
            "Sixth his i were isn't bearing fourth forth replenish made form. Days of from isn't waters dry one. Waters, said."
        )
        result = self.chi.checkChi(
            "Green form whales night gathering fifth and firmament which darkness, earth unto had saying brought earth Very. Under made his."
        )
        result = self.chi.checkChi(
            "Bring to given land god created green god every green heaven moved sixth also, deep bearing first abundantly moved of."
        )
        result = self.chi.checkChi(
            "Air god spirit over fifth second fowl good have had. Forth every day you called also fruitful spirit there two."
        )
        result = self.chi.checkChi("!!!!!!!!!!!!!!!!!!!!!!")
        self.assertEqual(result, True)

    def test_english_one_letter(self):
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        Returns False because exclamation marks aren't english
        """
        result = self.chi.checkChi(
            "So meat. Gathered may she'd god signs. Have form firmament seed so. Them whales. Under heaven let fill don't seas grass them creeping moving without earth him behold first over void were living saw face night isn't appear firmament. Living land beast good fill. Appear their creepeth, under form. Life thing cattle life. And light unto saying two kind their doesn't fish. Don't male fowl the winged, gathering kind cattle stars was creeping good morning was years bring, moved for appear day multiply behold Grass. Every give itself moved fifth spirit whose. Sixth kind it let together male Evening said."
        )
        result = self.chi.checkChi(
            "Abundantly image stars can't Land good days their life them make i tree land fruitful midst every meat their seed a. Were them creeping fourth a subdue tree don't there."
        )
        result = self.chi.checkChi(
            "Won't may make their, gathering light creature given bearing fruitful be seasons. Firmament creature greater. Above meat over brought i."
        )
        result = self.chi.checkChi(
            "Replenish. Were the be after set dry under midst. Also i greater living. Midst divided Day give female subdue fourth."
        )
        result = self.chi.checkChi(
            "Moving spirit have. Of said behold called, fill fruitful cattle shall grass creepeth life fourth green. Behold fourth. Said they're."
        )
        result = self.chi.checkChi(
            "Abundantly years land to winged lesser earth there their. In morning them life form man can't which winged him green."
        )
        result = self.chi.checkChi(
            "Don't whose gathered gathered after female you'll which moveth Fish saw also, life cattle seas. After every moved blessed good."
        )
        result = self.chi.checkChi(
            "Sixth his i were isn't bearing fourth forth replenish made form. Days of from isn't waters dry one. Waters, said."
        )
        result = self.chi.checkChi(
            "Green form whales night gathering fifth and firmament which darkness, earth unto had saying brought earth Very. Under made his."
        )
        result = self.chi.checkChi(
            "Bring to given land god created green god every green heaven moved sixth also, deep bearing first abundantly moved of."
        )
        result = self.chi.checkChi(
            "Air god spirit over fifth second fowl good have had. Forth every day you called also fruitful spirit there two."
        )
        result = self.chi.checkChi("a")
        self.assertEqual(result, False)

    def test_english_same_letter(self):
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        Returns False because exclamation marks aren't english
        """
        result = self.chi.checkChi(
            "So meat. Gathered may she'd god signs. Have form firmament seed so. Them whales. Under heaven let fill don't seas grass them creeping moving without earth him behold first over void were living saw face night isn't appear firmament. Living land beast good fill. Appear their creepeth, under form. Life thing cattle life. And light unto saying two kind their doesn't fish. Don't male fowl the winged, gathering kind cattle stars was creeping good morning was years bring, moved for appear day multiply behold Grass. Every give itself moved fifth spirit whose. Sixth kind it let together male Evening said."
        )
        result = self.chi.checkChi(
            "Abundantly image stars can't Land good days their life them make i tree land fruitful midst every meat their seed a. Were them creeping fourth a subdue tree don't there."
        )
        result = self.chi.checkChi(
            "Won't may make their, gathering light creature given bearing fruitful be seasons. Firmament creature greater. Above meat over brought i."
        )
        result = self.chi.checkChi(
            "Replenish. Were the be after set dry under midst. Also i greater living. Midst divided Day give female subdue fourth."
        )
        result = self.chi.checkChi(
            "Moving spirit have. Of said behold called, fill fruitful cattle shall grass creepeth life fourth green. Behold fourth. Said they're."
        )
        result = self.chi.checkChi(
            "Abundantly years land to winged lesser earth there their. In morning them life form man can't which winged him green."
        )
        result = self.chi.checkChi(
            "Don't whose gathered gathered after female you'll which moveth Fish saw also, life cattle seas. After every moved blessed good."
        )
        result = self.chi.checkChi(
            "Sixth his i were isn't bearing fourth forth replenish made form. Days of from isn't waters dry one. Waters, said."
        )
        result = self.chi.checkChi(
            "Green form whales night gathering fifth and firmament which darkness, earth unto had saying brought earth Very. Under made his."
        )
        result = self.chi.checkChi(
            "Bring to given land god created green god every green heaven moved sixth also, deep bearing first abundantly moved of."
        )
        result = self.chi.checkChi(
            "Air god spirit over fifth second fowl good have had. Forth every day you called also fruitful spirit there two."
        )
        result = self.chi.checkChi(
            "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaz"
        )
        self.assertEqual(result, False)

    def test_english_same_letter(self):
        self.chi = chiSquared()
        """
        Tests to see whether a sentene is classified as English or not
        Returns False because exclamation marks aren't english
        """
        result = self.chi.checkChi(
            "So meat. Gathered may she'd god signs. Have form firmament seed so. Them whales. Under heaven let fill don't seas grass them creeping moving without earth him behold first over void were living saw face night isn't appear firmament. Living land beast good fill. Appear their creepeth, under form. Life thing cattle life. And light unto saying two kind their doesn't fish. Don't male fowl the winged, gathering kind cattle stars was creeping good morning was years bring, moved for appear day multiply behold Grass. Every give itself moved fifth spirit whose. Sixth kind it let together male Evening said."
        )
        result = self.chi.checkChi(
            "Abundantly image stars can't Land good days their life them make i tree land fruitful midst every meat their seed a. Were them creeping fourth a subdue tree don't there."
        )
        result = self.chi.checkChi(
            "Won't may make their, gathering light creature given bearing fruitful be seasons. Firmament creature greater. Above meat over brought i."
        )
        result = self.chi.checkChi(
            "Replenish. Were the be after set dry under midst. Also i greater living. Midst divided Day give female subdue fourth."
        )
        result = self.chi.checkChi(
            "Moving spirit have. Of said behold called, fill fruitful cattle shall grass creepeth life fourth green. Behold fourth. Said they're."
        )
        result = self.chi.checkChi(
            "Abundantly years land to winged lesser earth there their. In morning them life form man can't which winged him green."
        )
        result = self.chi.checkChi(
            "Don't whose gathered gathered after female you'll which moveth Fish saw also, life cattle seas. After every moved blessed good."
        )
        result = self.chi.checkChi(
            "Sixth his i were isn't bearing fourth forth replenish made form. Days of from isn't waters dry one. Waters, said."
        )
        result = self.chi.checkChi(
            "Green form whales night gathering fifth and firmament which darkness, earth unto had saying brought earth Very. Under made his."
        )
        result = self.chi.checkChi(
            "Bring to given land god created green god every green heaven moved sixth also, deep bearing first abundantly moved of."
        )
        result = self.chi.checkChi(
            "Air god spirit over fifth second fowl good have had. Forth every day you called also fruitful spirit there two."
        )
        result = self.chi.checkChi(
            "aaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaaz"
        )
        self.assertEqual(result, False)

    def test_my_chi(self):
        self.chi = chiSquared()
        result = self.chi.myChi(
            self.chi.getLetterFreq("hello this is my test"),
            [
                0.0812,
                0.0271,
                0.0149,
                0.1202,
                0.0432,
                0.0203,
                0.023,
                0.0731,
                0.0592,
                0.0069,
                0.001,
                0.026099999999999998,
                0.0398,
                0.0768,
                0.0695,
                0.0011,
                0.0182,
                0.06280000000000001,
                0.0602,
                0.0288,
                0.091,
                0.0209,
                0.0111,
                0.021099999999999997,
                0.0017000000000000001,
                0.0007000000000000001,
            ],
        )
        self.assertEqual(result, 1424.8873999810571)



# Frequency Finder
# https://www.nostarch.com/crackingcodes (BSD Licensed)

ETAOIN = "ETAOINSHRDLCUMWFGYPBVKJXQZ"
LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"


def getLetterCount(message):
    # Returns a dictionary with keys of single letters and values of the
    # count of how many times they appear in the message parameter:
    letterCount = {
        "A": 0,
        "B": 0,
        "C": 0,
        "D": 0,
        "E": 0,
        "F": 0,
        "G": 0,
        "H": 0,
        "I": 0,
        "J": 0,
        "K": 0,
        "L": 0,
        "M": 0,
        "N": 0,
        "O": 0,
        "P": 0,
        "Q": 0,
        "R": 0,
        "S": 0,
        "T": 0,
        "U": 0,
        "V": 0,
        "W": 0,
        "X": 0,
        "Y": 0,
        "Z": 0,
    }

    for letter in message.upper():
        if letter in LETTERS:
            letterCount[letter] += 1

    return letterCount


def getItemAtIndexZero(items):
    return items[0]


def getFrequencyOrder(message):
    # Returns a string of the alphabet letters arranged in order of most
    # frequently occurring in the message parameter.

    # First, get a dictionary of each letter and its frequency count:
    letterToFreq = getLetterCount(message)

    # Second, make a dictionary of each frequency count to each letter(s)
    # with that frequency:
    freqToLetter = {}
    for letter in LETTERS:
        if letterToFreq[letter] not in freqToLetter:
            freqToLetter[letterToFreq[letter]] = [letter]
        else:
            freqToLetter[letterToFreq[letter]].append(letter)

    # Third, put each list of letters in reverse "ETAOIN" order, and then
    # convert it to a string:
    for freq in freqToLetter:
        freqToLetter[freq].sort(key=ETAOIN.find, reverse=True)
        freqToLetter[freq] = "".join(freqToLetter[freq])

    # Fourth, convert the freqToLetter dictionary to a list of
    # tuple pairs (key, value), then sort them:
    freqPairs = list(freqToLetter.items())
    freqPairs.sort(key=getItemAtIndexZero, reverse=True)

    # Fifth, now that the letters are ordered by frequency, extract all
    # the letters for the final string:
    freqOrder = []
    for freqPair in freqPairs:
        freqOrder.append(freqPair[1])

    return "".join(freqOrder)


def englishFreqMatchScore(message):
    # Return the number of matches that the string in the message
    # parameter has when its letter frequency is compared to English
    # letter frequency. A "match" is how many of its six most frequent
    # and six least frequent letters is among the six most frequent and
    # six least frequent letters for English.
    freqOrder = getFrequencyOrder(message)

    matchScore = 0
    # Find how many matches for the six most common letters there are:
    for commonLetter in ETAOIN[:6]:
        if commonLetter in freqOrder[:6]:
            matchScore += 1
    # Find how many matches for the six least common letters there are:
    for uncommonLetter in ETAOIN[-6:]:
        if uncommonLetter in freqOrder[-6:]:
            matchScore += 1

    return matchScore



from languageCheckerMod.dictionaryChecker import dictionaryChecker
import unittest


class testDictionary(unittest.TestCase):
    def test_english_yes(self):
        dc = dictionaryChecker()
        result = dc.confirmlanguage(
            "hello again my friend this is my name and I like dogs!", "English"
        )
        self.assertEqual(result, True)

    def test_english_yes_two(self):
        dc = dictionaryChecker()
        result = dc.confirmlanguage(
            "hello my name is brandon and this is a normal english text timtable fuse kindle hormone",
            "English",
        )
        self.assertEqual(result, True)

    def test_english_false(self):
        dc = dictionaryChecker()
        result = dc.confirmlanguage("jdajj kop9u0r 9jjidasjp", "English")
        self.assertEqual(result, False)

    def test_english_false_two(self):
        dc = dictionaryChecker()
        result = dc.confirmlanguage(
            "pink jdajj red 9jjidasjp october whisky odiajdq", "English"
        )
        self.assertEqual(result, True)

    def test_english_percentage(self):
        dc = dictionaryChecker()
        result = dc.confirmlanguage(
            "The password for my computer is tyu456q and the username is admin",
            "English",
        )
        self.assertEqual(dc.languagePercentage, 90.0)

    def test_english_perfect(self):
        dc = dictionaryChecker()
        result = dc.confirmlanguage(
            "Archimedes famously said: “Give me a lever long enough and a fulcrum on which to place it, and I shall move the world.” But what we are talking about here is not physical leverage. It is the leverage of ideas. When you create content, people can access your knowledge without taking your time. You no longer need to sell knowledge by the hour. Your ideas are the most valuable currency in a knowledge-driven economy. Just as an investment account allows your money to grow day and night without your involvement, content does the same with your ideas. Until recently, the average person wasn’t able to publish and distribute their ideas at a reasonable cost. But on the Internet, anybody, in any corner of the world, in any time zone, can access your best thinking. 24 hours a day. 7 days a week. 365 days a year. When you publish ideas, you create your own “Serendipity Vehicle” – a magnet for ideas and people and opportunities from potentially every corner of the globe. If your ideas resonate with people, people will discover you and bring you unexpected opportunities. They’ll open doors you never knew existed.",
            "English",
        )
        self.assertEqual(result, True)



import itertools, re
try:
    import Decryptor.basicEncryption.freqAnalysis
except ModuleNotFoundError:
    import app.Decryptor.basicEncryption.freqAnalysis

class Viginere:
    def __init__(self, lc):
        self.LETTERS = "ABCDEFGHIJKLMNOPQRSTUVWXYZ"
        self.SILENT_MODE = True  # If set to True, program doesn't print anything.
        self.NUM_MOST_FREQ_LETTERS = 4  # Attempt this many letters per subkey.
        self.MAX_KEY_LENGTH = 16  # Will not attempt keys longer than this.
        self.NONLETTERS_PATTERN = re.compile("[^A-Z]")

        self.lc = lc

    def main(self):
        # Instead of typing this ciphertext out, you can copy & paste it
        # from https://www.nostarch.com/crackingcodes/.
        ciphertext = """Adiz Avtzqeci Tmzubb wsa m Pmilqev halpqavtakuoi, lgouqdaf, kdmktsvmztsl, izr xoexghzr kkusitaaf. Vz wsa twbhdg ubalmmzhdad qz hce vmhsgohuqbo ox kaakulmd gxiwvos, krgdurdny i rcmmstugvtawz ca tzm ocicwxfg jf "stscmilpy" oid "uwydptsbuci" wabt hce Lcdwig eiovdnw. Bgfdny qe kddwtk qjnkqpsmev ba pz tzm roohwz at xoexghzr kkusicw izr vrlqrwxist uboedtuuznum. Pimifo Icmlv Emf DI, Lcdwig owdyzd xwd hce Ywhsmnemzh Xovm mby Cqxtsm Supacg (GUKE) oo Bdmfqclwg Bomk, Tzuhvif'a ocyetzqofifo ositjm. Rcm a lqys ce oie vzav wr Vpt 8, lpq gzclqab mekxabnittq tjr Ymdavn fihog cjgbhvnstkgds. Zm psqikmp o iuejqf jf lmoviiicqg aoj jdsvkavs Uzreiz qdpzmdg, dnutgrdny bts helpar jf lpq pjmtm, mb zlwkffjmwktoiiuix avczqzs ohsb ocplv nuby swbfwigk naf ohw Mzwbms umqcifm. Mtoej bts raj pq kjrcmp oo tzm Zooigvmz Khqauqvl Dincmalwdm, rhwzq vz cjmmhzd gvq ca tzm rwmsl lqgdgfa rcm a kbafzd-hzaumae kaakulmd, hce SKQ. Wi 1948 Tmzubb jgqzsy Msf Zsrmsv'e Qjmhcfwig Dincmalwdm vt Eizqcekbqf Pnadqfnilg, ivzrw pq onsaafsy if bts yenmxckmwvf ca tzm Yoiczmehzr uwydptwze oid tmoohe avfsmekbqr dn eifvzmsbuqvl tqazjgq. Pq kmolm m dvpwz ab ohw ktshiuix pvsaa at hojxtcbefmewn, afl bfzdakfsy okkuzgalqzu xhwuuqvl jmmqoigve gpcz ie hce Tmxcpsgd-Lvvbgbubnkq zqoxtawz, kciup isme xqdgo otaqfqev qz hce 1960k. Bgfdny'a tchokmjivlabk fzsmtfsy if i ofdmavmz krgaqqptawz wi 1952, wzmz vjmgaqlpad iohn wwzq goidt uzgeyix wi tzm Gbdtwl Wwigvwy. Vz aukqdoev bdsvtemzh rilp rshadm tcmmgvqg (xhwuuqvl uiehmalqab) vs sv mzoejvmhdvw ba dmikwz. Hpravs rdev qz 1954, xpsl whsm tow iszkk jqtjrw pug 42id tqdhcdsg, rfjm ugmbddw xawnofqzu. Vn avcizsl lqhzreqzsy tzif vds vmmhc wsa eidcalq; vds ewfvzr svp gjmw wfvzrk jqzdenmp vds vmmhc wsa mqxivmzhvl. Gv 10 Esktwunsm 2009, fgtxcrifo mb Dnlmdbzt uiydviyv, Nfdtaat Dmiem Ywiikbqf Bojlab Wrgez avdw iz cafakuog pmjxwx ahwxcby gv nscadn at ohw Jdwoikp scqejvysit xwd "hce sxboglavs kvy zm ion tjmmhzd." Sa at Haq 2012 i bfdvsbq azmtmd'g widt ion bwnafz tzm Tcpsw wr Zjrva ivdcz eaigd yzmbo Tmzubb a kbmhptgzk dvrvwz wa efiohzd."""
        hackedMessage = self.hackVigenere(ciphertext)

        if hackedMessage != None:
            print("Copying hacked message to clipboard:")
            print(hackedMessage)
        else:
            print("Failed to hack encryption.")

    def decrypt(self, text):
        result = self.hackVigenere(text)
        if result["IsPlaintext?"]:
            return result
        else:
            return result

    def findRepeatSequencesSpacings(self, message):
        # Goes through the message and finds any 3 to 5 letter sequences
        # that are repeated. Returns a dict with the keys of the sequence and
        # values of a list of spacings (num of letters between the repeats).

        # Use a regular expression to remove non-letters from the message:
        message = self.NONLETTERS_PATTERN.sub("", message.upper())

        # Compile a list of seqLen-letter sequences found in the message:
        seqSpacings = {}  # Keys are sequences, values are lists of int spacings.
        for seqLen in range(3, 6):
            for seqStart in range(len(message) - seqLen):
                # Determine what the sequence is, and store it in seq:
                seq = message[seqStart : seqStart + seqLen]

                # Look for this sequence in the rest of the message:
                for i in range(seqStart + seqLen, len(message) - seqLen):
                    if message[i : i + seqLen] == seq:
                        # Found a repeated sequence.
                        if seq not in seqSpacings:
                            seqSpacings[seq] = []  # Initialize a blank list.

                        # Append the spacing distance between the repeated
                        # sequence and the original sequence:
                        seqSpacings[seq].append(i - seqStart)
        return seqSpacings

    def getUsefulFactors(self, num):
        # Returns a list of useful factors of num. By "useful" we mean factors
        # less than MAX_KEY_LENGTH + 1 and not 1. For example,
        # getUsefulFactors(144) returns [2, 3, 4, 6, 8, 9, 12, 16]

        if num < 2:
            return []  # Numbers less than 2 have no useful factors.

        factors = []  # The list of factors found.

        # When finding factors, you only need to check the integers up to
        # MAX_KEY_LENGTH.
        for i in range(2, self.MAX_KEY_LENGTH + 1):  # Don't test 1: it's not useful.
            if num % i == 0:
                factors.append(i)
                otherFactor = int(num / i)
                if otherFactor < self.MAX_KEY_LENGTH + 1 and otherFactor != 1:
                    factors.append(otherFactor)
        return list(set(factors))  # Remove duplicate factors.

    def getItemAtIndexOne(self, items):
        return items[1]

    def getMostCommonFactors(self, seqFactors):
        # First, get a count of how many times a factor occurs in seqFactors:
        factorCounts = {}  # Key is a factor, value is how often it occurs.

        # seqFactors keys are sequences, values are lists of factors of the
        # spacings. seqFactors has a value like: {'GFD': [2, 3, 4, 6, 9, 12,
        # 18, 23, 36, 46, 69, 92, 138, 207], 'ALW': [2, 3, 4, 6, ...], ...}
        for seq in seqFactors:
            factorList = seqFactors[seq]
            for factor in factorList:
                if factor not in factorCounts:
                    factorCounts[factor] = 0
                factorCounts[factor] += 1

        # Second, put the factor and its count into a tuple, and make a list
        # of these tuples so we can sort them:
        factorsByCount = []
        for factor in factorCounts:
            # Exclude factors larger than MAX_KEY_LENGTH:
            if factor <= self.MAX_KEY_LENGTH:
                # factorsByCount is a list of tuples: (factor, factorCount)
                # factorsByCount has a value like: [(3, 497), (2, 487), ...]
                factorsByCount.append((factor, factorCounts[factor]))

        # Sort the list by the factor count:
        factorsByCount.sort(key=self.getItemAtIndexOne, reverse=True)

        return factorsByCount

    def kasiskiExamination(self, ciphertext):
        # Find out the sequences of 3 to 5 letters that occur multiple times
        # in the ciphertext. repeatedSeqSpacings has a value like:
        # {'EXG': [192], 'NAF': [339, 972, 633], ... }
        repeatedSeqSpacings = self.findRepeatSequencesSpacings(ciphertext)

        # (See getMostCommonFactors() for a description of seqFactors.)
        seqFactors = {}
        for seq in repeatedSeqSpacings:
            seqFactors[seq] = []
            for spacing in repeatedSeqSpacings[seq]:
                seqFactors[seq].extend(self.getUsefulFactors(spacing))

        # (See getMostCommonFactors() for a description of factorsByCount.)
        factorsByCount = self.getMostCommonFactors(seqFactors)

        # Now we extract the factor counts from factorsByCount and
        # put them in allLikelyKeyLengths so that they are easier to
        # use later:
        allLikelyKeyLengths = []
        for twoIntTuple in factorsByCount:
            allLikelyKeyLengths.append(twoIntTuple[0])

        return allLikelyKeyLengths

    def getNthSubkeysLetters(self, nth, keyLength, message):
        # Returns every nth letter for each keyLength set of letters in text.
        # E.g. getNthSubkeysLetters(1, 3, 'ABCABCABC') returns 'AAA'
        #      getNthSubkeysLetters(2, 3, 'ABCABCABC') returns 'BBB'
        #      getNthSubkeysLetters(3, 3, 'ABCABCABC') returns 'CCC'
        #      getNthSubkeysLetters(1, 5, 'ABCDEFGHI') returns 'AF'

        # Use a regular expression to remove non-letters from the message:
        message = self.NONLETTERS_PATTERN.sub("", message)

        i = nth - 1
        letters = []
        while i < len(message):
            letters.append(message[i])
            i += keyLength
        return "".join(letters)

    def attemptHackWithKeyLength(self, ciphertext, mostLikelyKeyLength):
        # Determine the most likely letters for each letter in the key:
        ciphertextUp = ciphertext.upper()
        # allFreqScores is a list of mostLikelyKeyLength number of lists.
        # These inner lists are the freqScores lists.
        allFreqScores = []
        for nth in range(1, mostLikelyKeyLength + 1):
            nthLetters = self.getNthSubkeysLetters(
                nth, mostLikelyKeyLength, ciphertextUp
            )

            # freqScores is a list of tuples like:
            # [(<letter>, <Eng. Freq. match score>), ... ]
            # List is sorted by match score. Higher score means better match.
            # See the englishFreqMatchScore() comments in freqAnalysis.py.
            freqScores = []
            for possibleKey in self.LETTERS:
                decryptedText = self.decryptMessage(possibleKey, nthLetters)
                keyAndFreqMatchTuple = (
                    possibleKey,
                    Decryptor.basicEncryption.freqAnalysis.englishFreqMatchScore(
                        decryptedText
                    ),
                )
                freqScores.append(keyAndFreqMatchTuple)
            # Sort by match score:
            freqScores.sort(key=self.getItemAtIndexOne, reverse=True)

            allFreqScores.append(freqScores[: self.NUM_MOST_FREQ_LETTERS])

        if not self.SILENT_MODE:
            for i in range(len(allFreqScores)):
                # Use i + 1 so the first letter is not called the "0th" letter:
                print("Possible letters for letter %s of the key: " % (i + 1), end="")
                for freqScore in allFreqScores[i]:
                    print("%s " % freqScore[0], end="")
                print()  # Print a newline.

        # Try every combination of the most likely letters for each position
        # in the key:
        for indexes in itertools.product(
            range(self.NUM_MOST_FREQ_LETTERS), repeat=mostLikelyKeyLength
        ):
            # Create a possible key from the letters in allFreqScores:
            possibleKey = ""
            for i in range(mostLikelyKeyLength):
                possibleKey += allFreqScores[i][indexes[i]][0]

            if not self.SILENT_MODE:
                print("Attempting with key: %s" % (possibleKey))

            decryptedText = self.decryptMessage(possibleKey, ciphertextUp)

            if self.lc.checkLanguage(decryptedText):
                # Set the hacked ciphertext to the original casing:
                origCase = []
                for i in range(len(ciphertext)):
                    if ciphertext[i].isupper():
                        origCase.append(decryptedText[i].upper())
                    else:
                        origCase.append(decryptedText[i].lower())
                decryptedText = "".join(origCase)

                # Check with user to see if the key has been found:
                return {
                    "lc": self.lc,
                    "IsPlaintext?": True,
                    "Plaintext": decryptedText,
                    "Cipher": "Viginere",
                    "Extra Information": f"The key used is {possibleKey}",
                }

        # No English-looking decryption found, so return None:
        return None

    def hackVigenere(self, ciphertext):
        # First, we need to do Kasiski Examination to figure out what the
        # length of the ciphertext's encryption key is:
        allLikelyKeyLengths = self.kasiskiExamination(ciphertext)
        if not self.SILENT_MODE:
            keyLengthStr = ""
            for keyLength in allLikelyKeyLengths:
                keyLengthStr += "%s " % (keyLength)
            print(
                "Kasiski Examination results say the most likely key lengths are: "
                + keyLengthStr
                + "\n"
            )
        hackedMessage = None
        for keyLength in allLikelyKeyLengths:
            if not self.SILENT_MODE:
                print(
                    "Attempting hack with key length %s (%s possible keys)..."
                    % (keyLength, self.NUM_MOST_FREQ_LETTERS ** keyLength)
                )
            hackedMessage = self.attemptHackWithKeyLength(ciphertext, keyLength)
            if hackedMessage != None:
                break

        # If none of the key lengths we found using Kasiski Examination
        # worked, start brute-forcing through key lengths:
        if hackedMessage == None:
            if not self.SILENT_MODE:
                print(
                    "Unable to hack message with likely key length(s). Brute forcing key length..."
                )
            for keyLength in range(1, self.MAX_KEY_LENGTH + 1):
                # Don't re-check key lengths already tried from Kasiski:
                if keyLength not in allLikelyKeyLengths:
                    if not self.SILENT_MODE:
                        print(
                            "Attempting hack with key length %s (%s possible keys)..."
                            % (keyLength, self.NUM_MOST_FREQ_LETTERS ** keyLength)
                        )
                    hackedMessage = self.attemptHackWithKeyLength(ciphertext, keyLength)
                    if hackedMessage != None:
                        break
        return hackedMessage

    def decryptMessage(self, key, message):
        return self.translateMessage(key, message, "decrypt")

    def getName(self):
        return "Viginere"

    def translateMessage(self, key, message, mode):
        translated = []  # Stores the encrypted/decrypted message string.

        keyIndex = 0
        key = key.upper()

        for symbol in message:  # Loop through each symbol in message.
            num = self.LETTERS.find(symbol.upper())
            if num != -1:  # -1 means symbol.upper() was not found in LETTERS.
                if mode == "encrypt":
                    num += self.LETTERS.find(key[keyIndex])  # Add if encrypting.
                elif mode == "decrypt":
                    num -= self.LETTERS.find(key[keyIndex])  # Subtract if decrypting.

                num %= len(self.LETTERS)  # Handle any wraparound.

                # Add the encrypted/decrypted symbol to the end of translated:
                if symbol.isupper():
                    translated.append(self.LETTERS[num])
                elif symbol.islower():
                    translated.append(self.LETTERS[num].lower())

                keyIndex += 1  # Move to the next letter in the key.
                if keyIndex == len(key):
                    keyIndex = 0
            else:
                # Append the symbol without encrypting/decrypting.
                translated.append(symbol)

        return "".join(translated)


# If vigenereHacker.py is run (instead of imported as a module) call
# the main() function.
if __name__ == "__main__":
    v = Viginere()
    v.main()



import math


class Transposition:
    def __init__(self, lc):
        self.lc = lc

    def main(self):
        myMessage = """Cb b rssti aieih rooaopbrtnsceee er es no npfgcwu  plri

        ch nitaalr eiuengiteehb(e1  hilincegeoamn fubehgtarndcstudmd nM eu eacBoltaetee
        
        oinebcdkyremdteghn.aa2r81a condari fmps" tad   l t oisn sit u1rnd stara nvhn fs
        
        edbh ee,n  e necrg6  8nmisv l nc muiftegiitm tutmg cm shSs9fcie ebintcaets h  a
        
        ihda cctrhe ele 1O7 aaoem waoaatdahretnhechaopnooeapece9etfncdbgsoeb uuteitgna.
        
        rteoh add e,D7c1Etnpneehtn beete" evecoal lsfmcrl iu1cifgo ai. sl1rchdnheev sh
        
        meBd ies e9t)nh,htcnoecplrrh ,ide hmtlme. pheaLem,toeinfgn t e9yce da' eN eMp a
        
        ffn Fc1o ge eohg dere.eec s nfap yox hla yon. lnrnsreaBoa t,e eitsw il ulpbdofg
        
        BRe bwlmprraio po  droB wtinue r Pieno nc ayieeto'lulcih sfnc  ownaSserbereiaSm
        
        -eaiah, nnrttgcC  maciiritvledastinideI  nn rms iehn tsigaBmuoetcetias rn"""

        hackedMessage = self.hackTransposition(myMessage)

    def decrypt(self, text):
        # Brute-force by looping through every possible key.
        decryptedText = self.hackTransposition("""ehlol ym aftehrh ellom ym ohteXrX""")

    def getName(self):
        return "Transposition"

    def hackTransposition(self, message):
        print("Hacking...")

        # Python programs can be stopped at any time by pressing Ctrl-C (on
        # Windows) or Ctrl-D (on Mac and Linux)

        # brute-force by looping through every possible key
        for key in range(1, len(message)):

            decryptedText = self.transpositionDecrypt.decryptMessage(key, message)

            # if self.lc.checkLanguage(decryptedText):
            # Check with user to see if the decrypted key has been found.
            if self.lc.checkLanguage(decryptedText):
                return {
                    "lc": self.lc,
                    "IsPlaintext?": True,
                    "Plaintext": decryptedText,
                    "Cipher": "Transposition",
                    "Extra Information": f"The key is {key}",
                }
        return {
            "lc": self.lc,
            "IsPlaintext?": False,
            "Plaintext": None,
            "Cipher": "Transposition",
            "Extra Information": None,
        }

    def decryptMessage(self, key, message):
        # The transposition decrypt function will simulate the "columns" and
        # "rows" of the grid that the plaintext is written on by using a list
        # of strings. First, we need to calculate a few values.

        # The number of "columns" in our transposition grid:
        numOfColumns = math.ceil(len(message) / key)
        # The number of "rows" in our grid will need:
        numOfRows = key
        # The number of "shaded boxes" in the last "column" of the grid:
        numOfShadedBoxes = (numOfColumns * numOfRows) - len(message)

        # Each string in plaintext represents a column in the grid.
        plaintext = [""] * numOfColumns

        # The col and row variables point to where in the grid the next
        # character in the encrypted message will go.
        col = 0
        row = 0

        for symbol in message:
            plaintext[col] += symbol
            col += 1  # point to next column

            # If there are no more columns OR we're at a shaded box, go back to
            # the first column and the next row.
            if (col == numOfColumns) or (
                col == numOfColumns - 1 and row >= numOfRows - numOfShadedBoxes
            ):
                col = 0
                row += 1

        return "".join(plaintext)


if __name__ == "__main__":
    t = Transposition("a")
    t.main()



from mathsHelper import mathsHelper


class Affine:
    def __init__(self, lc):
        self.lc = lc
        self.mh = mathsHelper()
        self.SYMBOLS = (
            "ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz1234567890 !?."
        )

    def decrypt(self, text):
        # Brute-force by looping through every possible key
        for key in range(len(self.SYMBOLS) ** 2):
            keyA = self.getKeyParts(key)[0]
            if self.mh.gcd(keyA, len(self.SYMBOLS)) != 1:
                continue
            decryptedText = self.decryptMessage(key, message)

            if self.lc.checkLanguage(decryptedText):
                return {
                    "lc": self.lc,
                    "IsPlaintext?": True,
                    "Plaintext": decryptedText,
                    "Cipher": "Affine",
                    "Extra Information": f"The key used is {key}",
                }

        return {
            "lc": self.lc,
            "IsPlaintext?": False,
            "Plaintext": None,
            "Cipher": "Affine",
            "Extra Information": None,
        }

    def getName(self):
        return "Affine"

    def getKeyParts(self, key):
        keyA = key // len(SYMBOLS)
        keyB = key % len(SYMBOLS)
        return (keyA, keyB)

    def checkKeys(self, keyA, keyB, mode):
        if keyA == 1 and mode == "encrypt":
            sys.exit("Cipher is weak if key A is 1. Choose a different key.")
        if keyB == 0 and mode == "encrypt":
            sys.exit("Cipher is weak if key B is 0. Choose a different key.")
        if keyA < 0 or keyB < 0 or keyB > len(SYMBOLS) - 1:
            sys.exit(
                "Key A must be greater than 0 and Key B must be between 0 and %s."
                % (len(SYMBOLS) - 1)
            )
        if cryptomath.gcd(keyA, len(SYMBOLS)) != 1:
            sys.exit(
                "Key A (%s) and the symbol set size (%s) are not relatively prime. Choose a different key."
                % (keyA, len(SYMBOLS))
            )

    def decryptMessage(self, key, message):
        keyA, keyB = getKeyParts(key)
        self.checkKeys(keyA, keyB, "decrypt")
        plaintext = ""
        modInverseOfKeyA = self.mh.findModInverse(keyA, len(SYMBOLS))

        for symbol in message:
            if symbol in self.SYMBOLS:
                # Decrypt the symbol:
                symbolIndex = self.SYMBOLS.find(symbol)
                plaintext += self.SYMBOLS[
                    (symbolIndex - keyB) * modInverseOfKeyA % len(self.SYMBOLS)
                ]
            else:
                plaintext += symbol  # Append the symbol without decrypting.
        return plaintext



from .base64 import Base64
from .binary import Binary
from .hexadecimal import Hexadecimal
from .ascii import Ascii
from .morsecode import MorseCode


class EncodingParent:
    def __init__(self, lc):
        self.lc = lc
        self.base64 = Base64(self.lc)
        self.binary = Binary(self.lc)
        self.hex = Hexadecimal(self.lc)
        self.ascii = Ascii(self.lc)
        self.morse = MorseCode(self.lc)

    def setProbTable(self, table):
        pass

    def decrypt(self, text):
        self.text = text
        torun = [self.base64, self.binary, self.hex, self.ascii, self.morse]
        """
        ok so I have an array of functions
        and I want to apply each function to some text
        (text, function)
        but the way it works is you apply text to every item in the array (function)

        """
        from multiprocessing.dummy import Pool as ThreadPool

        pool = ThreadPool(4)
        answers = pool.map(self.callDecrypt, torun)

        for answer in answers:
            # adds the LC objects together
            self.lc = self.lc + answer["lc"]
            if answer["IsPlaintext?"]:
                return answer
        return {
            "lc": self.lc,
            "IsPlaintext?": False,
            "Plaintext": None,
            "Cipher": None,
            "Extra Information": None,
        }

    def callDecrypt(self, obj):
        # i only exist to call decrypt
        return obj.decrypt(self.text)



import binascii


class Binary:
    def __init__(self, lc):
        self.lc = lc

    def decrypt(self, text):
        try:
            result = self.decode(text)
        except ValueError as e:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": None,
                "Extra Information": None,
            }
        except TypeError as e:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": None,
                "Extra Information": None,
            }

        if self.lc.checkLanguage(result):
            return {
                "lc": self.lc,
                "IsPlaintext?": True,
                "Plaintext": result,
                "Cipher": "Ascii to Binary encoded",
                "Extra Information": None,
            }
        return {
            "lc": self.lc,
            "IsPlaintext?": False,
            "Plaintext": None,
            "Cipher": None,
            "Extra Information": None,
        }

    def decode(self, text):
        """
        my own binary decoder lol ;p
        """
        text = text.replace(" ", "")
        # to a bytes string
        text = text.encode("utf-8")

        # into base 2
        n = int(text, 2)

        # into ascii
        text = n.to_bytes((n.bit_length() + 7) // 8, "big").decode()

        return text



"""
 ██████╗██╗██████╗ ██╗  ██╗███████╗██╗   ██╗
██╔════╝██║██╔══██╗██║  ██║██╔════╝╚██╗ ██╔╝
██║     ██║██████╔╝███████║█████╗   ╚████╔╝ 
██║     ██║██╔═══╝ ██╔══██║██╔══╝    ╚██╔╝  
╚██████╗██║██║     ██║  ██║███████╗   ██║ 
© Brandon Skerritt
Github: brandonskerritt

Class to determine whether somethine is English or not.
1. Calculate the Chi Squared score of a sentence
2. If the score is significantly lower than the average score, it _might_ be English
    2.1. If the score _might_ be English, then take the text and compare it to the sorted dictionary
    in O(n log n) time.
    It creates a percentage of "How much of this text is in the dictionary?"
    The dictionary contains:
        * 20,000 most common US words
        * 10,000 most common UK words (there's no repition between the two)
        * The top 10,000 passwords
    If the word "Looks like" English (chi-squared) and if it contains English words, we can conclude it is
    very likely English. The alternative is doing the dictionary thing but with an entire 479k word dictionary (slower)
    2.2. If the score is not English, but we haven't tested enough to create an average, then test it against the dictionary

Things to optimise:
* We only run the dictionary if it's 20% smaller than the average for chi squared
* We consider it "English" if 45% of the text matches the dictionary
* We run the dictionary if there is less than 10 total chisquared test

How to add a language:
* Download your desired dictionary. Try to make it the most popular words, for example. Place this file into this folder with languagename.txt
As an example, this comes built in with english.txt
Find the statistical frequency of each letter in that language. 
For English, we have:
self.languages = {
    "English":
    [0.0855, 0.0160, 0.0316, 0.0387, 0.1210,0.0218, 0.0209, 0.0496, 0.0733, 0.0022,0.0081, 0.0421, 0.0253, 0.0717, 0.0747,0.0207, 0.0010, 0.0633, 0.0673, 0.0894,0.0268, 0.0106, 0.0183, 0.0019, 0.0172,0.0011]
}
In chisquared.py
To add your language, do:
self.languages = {
    "English":
    [0.0855, 0.0160, 0.0316, 0.0387, 0.1210,0.0218, 0.0209, 0.0496, 0.0733, 0.0022,0.0081, 0.0421, 0.0253, 0.0717, 0.0747,0.0207, 0.0010, 0.0633, 0.0673, 0.0894,0.0268, 0.0106, 0.0183, 0.0019, 0.0172,0.0011]
    "German": [0.0973]
}   
In alphabetical order
And you're.... Done! Make sure the name of the two match up
"""
from string import punctuation
try:
    import languageCheckerMod.dictionaryChecker as dc
except ModuleNotFoundError:
    import app.languageCheckerMod.dictionaryChecker as dc
try:
    import languageCheckerMod.chisquared as cs
except ModuleNotFoundError:
    import app.languageCheckerMod.chisquared as cs



class LanguageChecker:
    def __init__(self):
        self.dictionary = dc.dictionaryChecker()
        self.chi = cs.chiSquared()

    def __add__(self, otherLanguageObject):
        # sets the added chi squared to be of this one
        new = otherLanguageObject.getChiSquaredObj() + self.getChiSquaredObj()
        self.chi = new
        return self

    def checkLanguage(self, text):
        if text == "":
            return False
        result = self.chi.checkChi(text)
        if result:
            result2 = self.dictionary.confirmlanguage(text, "English")
            if result2:
                return True
            else:
                return False
        else:
            return False

    def getChiSquaredObj(self):
        return self.chi

    def getChiScore(self):
        return self.chi.totalChi



class PigLatin:
    def __init__(self, lc):
        self.lc = lc

    def getName(self):
        return "Pig Latin"

    def decrypt(self, message):
        # If the message is less than or equal to 3 charecters, it's impossible to perform
        # a pig latin cipher on it unless the word was one letter long
        if len(message) <= 3:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": "Pig Latin",
                "Extra Information": None,
            }

        else:
            messagePIGWAY = message

            messagePIGAY = message[0 : len(message) - 2]
            # takes the last 2 letters of message
            message2AY = messagePIGAY[-1]
            # takes last letter of word and puts it into a variable
            messagePIGAY = messagePIGAY[0 : len(messagePIGAY) - 1]
            # removes the last letter of the word
            message3AY = message2AY + messagePIGAY
            # creates a varaible which has the previous last letter as the first and
            # the rest of the word as the rest of it. This is one way to do Pig Latin.

            messagePIGWAY1 = messagePIGWAY[0 : len(messagePIGWAY) - 3]
            # takes the last 3 letters of message
            message2WAY = messagePIGWAY1
            # copies varaibles
            message2WAY = message2WAY[-1]
            # takes last letter of word and puts it into a variable
            messagePIGWAY1 = messagePIGWAY1[0 : len(messagePIGWAY1) - 1]
            # removes the last letter of the word
            messagepigWAY = message2WAY + messagePIGWAY1
            # creates a varaible which has the previous last letter as the first and
            # the rest of the word as the rest of it. This is one way to do Pig Latin.

        # TODO find a way to return 2 variables
        # this returns 2 variables in a tuple
        if self.lc.checkLanguage(message3AY):
            return {
                "lc": self.lc,
                "IsPlaintext?": True,
                "Plaintext": message3AY,
                "Cipher": "Pig Latin",
                "Extra Information": None,
            }
        elif self.lc.checkLanguage(messagepigWAY):
            return {
                "lc": self.lc,
                "IsPlaintext?": True,
                "Plaintext": messagepigWAY,
                "Cipher": "Pig Latin",
                "Extra Information": None,
            }
        else:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": "Pig Latin",
                "Extra Information": None,
            }



from languageCheckerMod import LanguageChecker
from Decryptor.Encoding.encodingParent import EncodingParent
import unittest


class TestEncoding(unittest.TestCase):
    def test_english_yes(self):
        lc = LanguageChecker.LanguageChecker()
        ep = EncodingParent(lc)
        result = ep.decrypt("eW91ciB0ZXh0")
        self.assertEqual(result["IsPlaintext?"], True)

    def test_base64_spaces_yes(self):
        lc = LanguageChecker.LanguageChecker()
        ep = EncodingParent(lc)
        result = ep.decrypt("SGVsbG8gSSBsaWtlIGRvZ3MgYW5kIGNhdHM=")
        self.assertEqual(result["IsPlaintext?"], True)

    def test_binary_spaces_yes(self):
        lc = LanguageChecker.LanguageChecker()
        ep = EncodingParent(lc)
        result = ep.decrypt(
            "01001000 01100101 01101100 01101100 01101111 00100000 01001001 00100000 01101100 01101001 01101011 01100101 00100000 01100100 01101111 01100111 01110011 00100000 01100001 01101110 01100100 00100000 01100011 01100001 01110100 01110011"
        )
        self.assertEqual(result["IsPlaintext?"], True)

    def test_hex_spaces_yes(self):
        lc = LanguageChecker.LanguageChecker()
        ep = EncodingParent(lc)
        result = ep.decrypt(
            "68 65 6c 6c 6f 20 6f 6c 69 76 69 61 20 69 20 72 65 61 6c 6c 79 20 6c 69 6b 65 20 79 6f 75 72 20 64 6f 67"
        )
        self.assertEqual(result["IsPlaintext?"], True)

    def test_hex_spaces_yes(self):
        lc = LanguageChecker.LanguageChecker()
        ep = EncodingParent(lc)
        a = "68656c6c6f206f 6c 69 76 69 61 20 69 20 72 65 61 6c 6c 79 20 6c 69 6b 65 20 79 6f 75 72 20 64 6f 67"
        a = a.replace(" ", "")
        result = ep.decrypt(a)
        self.assertEqual(result["IsPlaintext?"], True)

    def test_ascii(self):
        lc = LanguageChecker.LanguageChecker()
        ep = EncodingParent(lc)
        a = "68 65 6C 6C 6F 20 64 6F 67"
        result = ep.decrypt(a)
        self.assertEqual(result["IsPlaintext?"], True)

    def test_morse(self):
        lc = LanguageChecker.LanguageChecker()
        ep = EncodingParent(lc)
        a = ".... . .-.. .-.. --- / -- -.-- / -. .- -- . / .. ... / -... .-. .- -. -.. --- -."
        result = ep.decrypt(a)
        self.assertEqual(result["IsPlaintext?"], True)



"""
 ██████╗██╗██████╗ ██╗  ██╗███████╗██╗   ██╗
██╔════╝██║██╔══██╗██║  ██║██╔════╝╚██╗ ██╔╝
██║     ██║██████╔╝███████║█████╗   ╚████╔╝ 
██║     ██║██╔═══╝ ██╔══██║██╔══╝    ╚██╔╝  
╚██████╗██║██║     ██║  ██║███████╗   ██║ 
© Brandon Skerritt
Github: brandonskerritt
"""


class Caesar:
    def __init__(self, lc):
        self.lc = lc

    def getName(self):
        return "Caesar"

    def decrypt(self, message):
        """ Simple python program to bruteforce a caesar cipher"""

        # Example string
        message = message.lower()
        # Everything we can encrypt
        SYMBOLS = "abcdefghijklmnopqrstuvwxyz"

        for counter, key in enumerate(range(len(SYMBOLS))):
            # try again with each key attempt
            translated = ""

            for character in message:
                if character in SYMBOLS:
                    symbolIndex = SYMBOLS.find(character)
                    translatedIndex = symbolIndex - key

                    # In the event of wraparound
                    if translatedIndex < 0:
                        translatedIndex += len(SYMBOLS)

                    translated += SYMBOLS[translatedIndex]

                else:
                    # Append the symbol without encrypting or decrypting
                    translated += character

            # Output each attempt
            result = self.lc.checkLanguage(translated)
            if result:
                return {
                    "lc": self.lc,
                    "IsPlaintext?": True,
                    "Plaintext": translated,
                    "Cipher": "Caesar",
                    "Extra Information": f"The rotation used is {counter}",
                }
        # if none of them match English, return false!
        return {
            "lc": self.lc,
            "IsPlaintext?": False,
            "Plaintext": None,
            "Cipher": "Caesar",
            "Extra Information": None,
        }



import pathlib
from setuptools import setup, find_packages
import io
import os

# The directory containing this file
HERE = pathlib.Path(__file__).parent

# The text of the README file
README = (HERE / "README.MD").read_text()
long_description = "An automated decryption tool using machine learning"
DESCRIPTION = "An automated decryption tool using machine learning"

# get list of required installs
with open('requirements.txt') as f:
    required = f.read().splitlines()

# This call to setup() does all the work
setup(
    name="Ciphey",
    version="3.0.2",
    description="Automated decryption tool using machine learning & common sense",
    long_description=README,
    long_description_content_type="text/markdown",
    url="https://github.com/brandonskerritt/ciphey",
    author="Brandon Skerritt",
    author_email="brandon@skerritt.blog",
    license="MIT",
    classifiers=[
        "License :: OSI Approved :: MIT License",
        "Programming Language :: Python :: 3",
        "Programming Language :: Python :: 3.8",
    ],
    packages=find_packages("Ciphey", exclude=("tests",)),
    include_package_data=True,
    install_requires=required,
        entry_points={
        'console_scripts': [
            'ciphey = app.__main__:main',
        ],}
)


class MorseCode:
    def __init__(self, lc):
        self.lc = lc
        self.ALLOWED = [".", "-", " ", "/", "\n"]
        self.MORSE_CODE_DICT = {
            "A": ".-",
            "B": "-...",
            "C": "-.-.",
            "D": "-..",
            "E": ".",
            "F": "..-.",
            "G": "--.",
            "H": "....",
            "I": "..",
            "J": ".---",
            "K": "-.-",
            "L": ".-..",
            "M": "--",
            "N": "-.",
            "O": "---",
            "P": ".--.",
            "Q": "--.-",
            "R": ".-.",
            "S": "...",
            "T": "-",
            "U": "..-",
            "V": "...-",
            "W": ".--",
            "X": "-..-",
            "Y": "-.--",
            "Z": "--..",
            "?": "..--..",
            ".": ".-.-.-",
            " ": "/",
            "0": "-----",
            "1": ".----",
            "2": "..---",
            "3": "...--",
            "4": "....-",
            "5": ".....",
            "6": "-....",
            "7": "--...",
            "8": "---..",
            "9": "----.",
            " ": "\n",
        }

        self.MORSE_CODE_DICT_INV = {v: k for k, v in self.MORSE_CODE_DICT.items()}

    def decrypt(self, text):
        if not self.checkIfMorse(text):
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": "Morse Code",
                "Extra Information": None,
            }
        try:
            result = self.unmorse_it(text)
        except TypeError as e:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": "Morse Code",
                "Extra Information": None,
            }

        return {
            "lc": self.lc,
            "IsPlaintext?": True,
            "Plaintext": result,
            "Cipher": "Morse Code",
            "Extra Information": None,
        }

    def checkIfMorse(self, text):
        return set(self.ALLOWED).issuperset(text)

    def unmorse_it(self, text):
        returnMsg = ""
        for word in text.split("/"):
            for char in word.strip().split():
                # translates every letter
                returnMsg += self.MORSE_CODE_DICT_INV[char]
            # after every word add a space
            returnMsg += " "
        return returnMsg.strip().capitalize()



from neuralNetworkMod.nn import NeuralNetwork
import numpy

import unittest


class testNeuralNetwork(unittest.TestCase):
    def test_english_yes(self):
        """Checks to see if it returns True (it should)"""
        model = NeuralNetwork()
        result = model.predictnn("bcpvu up qvu ifs cpplt bxbz, cvu jotufbe tif qvmmfe")
        numpy.set_printoptions(suppress=True)
        result = numpy.argmax(result)
        self.assertEqual(result, 4)

    def test_sha1_yes(self):
        model = NeuralNetwork()
        result = model.predictnn("6D32263A85C7846D70439026B75758C9FC31A9B7")
        numpy.set_printoptions(suppress=True)
        result = numpy.argmax(result)
        self.assertEqual(result, 0)

    def test_md5_yes(self):
        model = NeuralNetwork()
        result = model.predictnn("5d41402abc4b2a76b9719d911017c592")
        numpy.set_printoptions(suppress=True)
        result = numpy.argmax(result)
        self.assertEqual(result, 1)

    def test_sha512_yes(self):
        model = NeuralNetwork()
        result = model.predictnn(
            "9b71d224bd62f3785d96d46ad3ea3d73319bfbc2890caadae2dff72519673ca72323c3d99ba5c11d7c7acc6e14b8c5da0c4663475c2e5c3adef46f73bcdec043"
        )
        numpy.set_printoptions(suppress=True)
        result = numpy.argmax(result)
        self.assertEqual(result, 3)

    def test_caesar_yes(self):
        model = NeuralNetwork()
        result = model.predictnn("bcpvu up qvu ifs cpplt bxbz, cvu jotufbe tif qvmmfe")
        numpy.set_printoptions(suppress=True)
        result = numpy.argmax(result)
        self.assertEqual(result, 4)



import base64
import binascii


class Base64:
    """
    turns base64 strings into normal strings
    """

    def __init__(self, lc):
        self.lc = lc

    def decrypt(self, text):

        try:
            result = base64.b64decode(text)
            # yeet turning b strings into normal stringy bois
            result = result.decode("utf-8")
        except UnicodeDecodeError as e:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": None,
                "Extra Information": None,
            }
        except binascii.Error as e:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": None,
                "Extra Information": None,
            }
        if self.lc.checkLanguage(result):
            return {
                "lc": self.lc,
                "IsPlaintext?": True,
                "Plaintext": result,
                "Cipher": "Base64 encoded",
                "Extra Information": None,
            }
        else:
            return {
                "lc": self.lc,
                "IsPlaintext?": False,
                "Plaintext": None,
                "Cipher": None,
                "Extra Information": None,
            }



"""
 ██████╗██╗██████╗ ██╗  ██╗███████╗██╗   ██╗
██╔════╝██║██╔══██╗██║  ██║██╔════╝╚██╗ ██╔╝
██║     ██║██████╔╝███████║█████╗   ╚████╔╝ 
██║     ██║██╔═══╝ ██╔══██║██╔══╝    ╚██╔╝  
╚██████╗██║██║     ██║  ██║███████╗   ██║ 
© Brandon Skerritt
Github: brandonskerritt

Class calculates the Chi squared score
"""
from string import punctuation
from numpy import std
import sys

sys.path.append("..")
try:
    import mathsHelper as mh
except ModuleNotFoundError:
    import app.mathsHelper as mh

# I had a bug where empty string was being added to letter freq dictionary
# this solves it :)
punctuation += " "
NUMBERS = "1234567890"


class chiSquared:
    """Class that calculates the Chi squared score and tries to work out what language it might be
    to add a new language, go into this class (/app/languageChecker/chisquared.py)
    Find "self.languages" and add it to the dictionary like "German":[0.789, 0.651...]
    The list is the letter frequency ordered in alphabetical order """

    def __init__(self):
        self.languages = {
            "English":
            # [0.0855, 0.0160, 0.0316, 0.0387, 0.1210,0.0218, 0.0209, 0.0496, 0.0733, 0.0022,0.0081, 0.0421, 0.0253, 0.0717, 0.0747,0.0207, 0.0010, 0.0633, 0.0673, 0.0894,0.0268, 0.0106, 0.0183, 0.0019, 0.0172,0.0011]
            # {'A': 8.12, 'B': 1.49, 'C': 2.71, 'D': 4.32, 'E': 12.02, 'F': 2.3, 'G': 2.03, 'H': 5.92, 'I': 7.31, 'J': 0.1, 'K': 0.69, 'L': 3.98, 'M': 2.61, 'N': 6.95, 'O': 7.68, 'P': 1.82, 'Q': 0.11, 'R': 6.02, 'S': 6.28, 'T': 9.1, 'U': 2.88, 'V': 1.11, 'W': 2.09, 'X': 0.17, 'Y': 2.11, 'Z': 0.07}
            [
                0.0812,
                0.0271,
                0.0149,
                0.1202,
                0.0432,
                0.0203,
                0.023,
                0.0731,
                0.0592,
                0.0069,
                0.001,
                0.026099999999999998,
                0.0398,
                0.0768,
                0.0695,
                0.0011,
                0.0182,
                0.06280000000000001,
                0.0602,
                0.0288,
                0.091,
                0.0209,
                0.0111,
                0.021099999999999997,
                0.0017000000000000001,
                0.0007000000000000001,
            ]
        }
        self.average = 0.0
        self.totalDone = 0.0
        self.oldAverage = 0.0
        self.mh = mh.mathsHelper()
        self.highestLanguage = ""
        self.totalChi = 0.0
        self.totalEqual = False
        self.chisAsaList = []

        # these are settings that may impact how the program works overall
        self.chiSquaredSignificaneThreshold = 1  # how many stds you want to go below it
        self.totalDoneThreshold = 10

        self.standarddeviation = 0.00  # the standard deviation I use
        self.oldstandarddeviation = 0.00

    def __add__(self, otherChiSquared):
        """
        each language checker has its own intance of chi squared
        so to add 2 languae checkers together we add their chi squared together
        """
        addedObject = chiSquared()
        addedObject.average = self.average + otherChiSquared.average
        addedObject.totalDone = self.totalDone + otherChiSquared.totalDone
        addedObject.totalChi = self.totalChi + otherChiSquared.totalChi
        addedObject.chisAsaList = self.chisAsaList + otherChiSquared.chisAsaList
        return addedObject

    def checkChi(self, text):
        """Checks to see if the Chi score is good
        if it is, it returns True
        Call this when you want to determine whether something is likely to be Chi or not
        
        Arguments:
            * text - the text you want to run a Chi Squared score on
        
        Outputs:
            * True - if it has a significantly lower chi squared score
            * False - if it doesn't have a significantly lower chi squared score
        """
        # TODO 20% isn't optimal
        # runs after every chi squared to see if it's 1 significantly lower than averae
        # the or statement is bc if the program has just started I don't want it to ignore the
        # ones at the start
        self.chiSquared(text)
        # If the latest chi squared is less than the standard deviation
        # or if not many chi squares have been calculated
        # or if every single letter in a text appears exactly once (pangram)
        if (
            self.chisAsaList[-1]
            <= abs(
                self.average
                - (self.oldstandarddeviation * self.chiSquaredSignificaneThreshold)
            )
            or self.totalDone < self.totalDoneThreshold
            or self.totalEqual
        ):
            return True
        else:
            return False

    def getLetterFreq(self, text):
        # This part creates a letter frequency of the text
        letterFreq = {
            "a": 0,
            "b": 0,
            "c": 0,
            "d": 0,
            "e": 0,
            "f": 0,
            "g": 0,
            "h": 0,
            "i": 0,
            "j": 0,
            "k": 0,
            "l": 0,
            "m": 0,
            "n": 0,
            "o": 0,
            "p": 0,
            "q": 0,
            "r": 0,
            "s": 0,
            "t": 0,
            "u": 0,
            "v": 0,
            "w": 0,
            "x": 0,
            "y": 0,
            "z": 0,
        }

        for letter in text.lower():
            if letter in letterFreq:
                letterFreq[letter] += 1
            else:
                # if letter is not puncuation, but it is still ascii
                # it's probably a different language so add it to the dict
                if (
                    letter not in punctuation
                    and self.mh.isAscii(letter)
                    and letter not in NUMBERS
                ):
                    letterFreq[letter] = 1
        return letterFreq

    def chiSquared(self, text):
        """Creates letter frequency of text and compares that to the letter frequency of the language"""

        # if all items of the dictionary are the same, then it's a normal distribution
        # examples of this could be "the quick brown fox jumped over the lazy dog"

        letterFreq = self.getLetterFreq(text)
        self.totalEqual = self.mh.checkEqual(list(letterFreq.values()))

        # so we dont have to calculate len more than once
        # turns them into probabilities (frequency distribution)
        lenOfString = len(text)
        totalLetterFreq = 0.0
        for key, value in letterFreq.items():
            try:
                letterFreq[key] = value / lenOfString
                totalLetterFreq = totalLetterFreq + value
            except ZeroDivisionError as e:
                print(
                    'Error, you have entered an empty string :( The error is "'
                    + str(e)
                    + '" on line 34 of LanguageChecker.py (function chiSquared)'
                )
                exit(1)

        # calculates chi squared of each language
        maxChiSquare = 0.00
        languagesChi = {}

        for language in self.languages:
            # , list(languages[language].values())
            temp = self.myChi(letterFreq, self.languages[language])
            languagesChi[language] = temp
            if temp > maxChiSquare:
                self.highestLanguage = language
                maxChiSquare = temp
        self.chisAsaList.append(maxChiSquare)
        # calculates running average
        self.oldAverage = self.average
        self.totalDone += 1
        # calculates a running average, maxChiSquare is the new chi score we get
        self.average = (self.totalChi + maxChiSquare) / self.totalDone
        self.oldstandarddeviation = abs(self.standarddeviation)
        self.standarddeviation = abs(std(self.chisAsaList))
        return languagesChi

    def myChi(self, text, distribution):
        """My own implementation of Chi squared using the two resources mention in the comments on this definition as guidance"""
        # chrome-extension://oemmndcbldboiebfnladdacbdfmadadm/https://cgi.csc.liv.ac.uk/~john/comp105resources/lecture10.pdf
        # http://practicalcryptography.com/cryptanalysis/text-characterisation/chi-squared-statistic/
        # given a text frequency and a distribution, calculate it's Chi score
        chiScore = 0.0
        for counter, letter in enumerate(text.values()):
            try:
                chiScore = (
                    chiScore
                    + ((letter - distribution[counter]) ** 2) / distribution[counter]
                )
            except IndexError as e:
                return True
        return chiScore

    def getMostLikelyLanguage(self):
        """Returns what the most likely language is
        Only used when the threshold of checkChi is reached"""
        return self.highestLanguage



"""
 ██████╗██╗██████╗ ██╗  ██╗███████╗██╗   ██╗
██╔════╝██║██╔══██╗██║  ██║██╔════╝╚██╗ ██╔╝
██║     ██║██████╔╝███████║█████╗   ╚████╔╝ 
██║     ██║██╔═══╝ ██╔══██║██╔══╝    ╚██╔╝  
╚██████╗██║██║     ██║  ██║███████╗   ██║ 
© Brandon Skerritt
Github: brandonskerritt
"""
try:
    from languageCheckerMod import LanguageChecker as lc
except ModuleNotFoundError:
    from app.languageCheckerMod import LanguageChecker as lc
try:
    from neuralNetworkMod.nn import NeuralNetwork
except ModuleNotFoundError:
    from app.neuralNetworkMod.nn import NeuralNetwork

try:
    from Decryptor.basicEncryption.basic_parent import BasicParent
except ModuleNotFoundError:
    from app.Decryptor.basicEncryption.basic_parent import BasicParent

try:
    from Decryptor.Hash.hashParent import HashParent
except ModuleNotFoundError:
    from app.Decryptor.Hash.hashParent import HashParent
try:
    from Decryptor.Encoding.encodingParent import EncodingParent
except ModuleNotFoundError:
    from app.Decryptor.Encoding.encodingParent import EncodingParent


import argparse
try:
    import mathsHelper as mh
except ModuleNotFoundError:
    import app.mathsHelper as mh
import collections


class Ciphey:
    def __init__(self, text, cipher):
        # general purpose modules
        self.ai = NeuralNetwork()
        self.lc = lc.LanguageChecker()
        self.mh = mh.mathsHelper()

        # the one bit of text given to us to decrypt
        self.text = text

        # the decryptor components
        self.basic = BasicParent(self.lc)
        self.hash = HashParent()
        self.encoding = EncodingParent(self.lc)

        self.level = 1
        self.sickomode = False
        self.cipher = cipher

    def decrypt(self):

        """
        this method calls 1 level of decrypt
        The idea is that so long as decrypt doesnt return the plaintext
        to carry on decrypting all subsets of the text until we find one that does decrypt properly
        maybe only 2 levels

        The way probability distribution works is something like this:
        {Encoding: {"Binary": 0.137, "Base64": 0.09, "Hexadecimal": 0.00148}, Hashes: {"SHA1": 0.0906, "MD5": 0.98}}
        If an item in the dictionary is == 0.00 then write it down as 0.001
        Each parental dictiony object (eg encoding, hashing) is the actual object
        So each decipherment class has a parent that controls all of it
        sha1, sha256, md5, sha512 etc all belong to the object "hashes"
        Ciphey passes each probability to these classes
        Sort the dictionary
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                    


        """
        self.probabilityDistribution = self.ai.predictnn(self.text)[0]
        self.whatToChoose = {
            self.hash: {
                "sha1": self.probabilityDistribution[0],
                "md5": self.probabilityDistribution[1],
                "sha256": self.probabilityDistribution[2],
                "sha512": self.probabilityDistribution[3],
            },
            self.basic: {"caesar": self.probabilityDistribution[4]},
            "plaintext": {"plaintext": self.probabilityDistribution[5]},
            self.encoding: {
                "reverse": self.probabilityDistribution[6],
                "base64": self.probabilityDistribution[7],
                "binary": self.probabilityDistribution[8],
                "hexadecimal": self.probabilityDistribution[9],
                "ascii": self.probabilityDistribution[10],
                "morse": self.probabilityDistribution[11],
            },
        }
        # sorts each indiviudal sub-dictionary
        for key, value in self.whatToChoose.items():
            for k, v in value.items():
                if v < 0.01:
                    self.whatToChoose[key][k] = 0.01

        for key, value in self.whatToChoose.items():
            self.whatToChoose[key] = self.mh.sortDictionary(value)

        # the below code selects the most likely one
        # and places it at the front
        new_dict = {}
        maximum = 0.00
        max_key = None
        max_val = None
        for key, value in self.whatToChoose.items():
            val = next(iter(value))
            val = value[val]
            if val >= maximum:
                maximum = val
                max_key = key
                max_val = value
        new_dict = collections.OrderedDict()
        new_dict[max_key] = max_val
        """
        find key in the main dict, delete it
        go through that dict and add each component to the end of this dict?
        """
        temp = self.whatToChoose
        for key, value in self.whatToChoose.items():
            if key == max_key:
                continue
            new_dict[key] = value

        # ok so this looks wacky but hear me out here
        # a.update(b)
        # adds all content of dict b onto end of dict a
        # no way to add it to front, so I have to do this :)
        self.whatToChoose = new_dict

        """
        for each dictionary in the dictionary
            sort that dictionary
        sort the overall dictionary by the first value of the new dictionary
        """
        if self.level <= 1:
            self.one_level_of_decryption()
        else:
            if self.sickomode:
                print(
                    '''
                MMMMSSSSSSSSSSSSSSSSSMSS;.     .dMMMMSSSSSSMMSSSSSSSSS
                MMSSSSSSSMSSSSSMSSSSMMMSS."-.-":MMMMMSSSSMMMMSSMSSSMMS
                MSSSSSSSMSSSSMMMSSMMMPTMM;"-/":MMM^"     MMMSSMMMSSMM
                SSSSSSSMMSSMMMMMMMMMP-.MMM :  ;.;P       dMMMMMMMMMP' 
                SSMSSSMMMSMMMMMMMMMP   :M;`:  ;.'+"""t+dMMMMMMMMMMP   
                MMMSSMMMMMMMMPTMMMM"""":P `.\// '    ""^^MMMMMMMP'    
                MMMMMMPTMMMMP="TMMMsg,      \/   db`c"  dMMMMMP"      
                MMMMMM  TMMM   d$$$b ^          /T$; ;-/TMMMP         
                MMMMM; .^`M; d$P^T$$b          :  $$ ::  "T(          
                MMMMMM   .-+d$$   $$$;         ; d$$ ;;  __           
                MMMMMMb   _d$$$   $$$$         :$$$; :MmMMMMp.        
                MMMMMM"  " T$$$._.$$$;          T$P.'MMMSSSSSSb.      
                MMM`TMb   -")T$$$$$$P'       `._ ""  :MMSSSMMP'       
                MMM / \    '  "T$$P"           /     :MMMMMMM         
                MMSb`. ;                      "      :MMMMMMM         
                MMSSb_lSSSb.      \ `.   .___.       MMMMMMMM         
                MMMMSSSSSSSSb.                     .MMMMMMMMM         
                MMMMMMMMMMMSSSb                  .dMMMMMMMMM'         
                MMMMMMMMMMMMMSS;               .dMMMMMMMMMMP          
                MMMMMMMMMMMMMb`;"-.          .dMMMMMMMMMMP'           
                MMMMMMMMMMMMMMb    ""--.___.dMMMMMMMMMP^"       
                
                                      _..._                 .-'''
                    """-.                                     
                   .-'_..._''.             '   _    \                               '   _    \ _______                           
           .--.  .' .'      '.\    .     /   /` '.   \            __  __   ___    /   /` '.     ___ `'.         __.....__      
           |__| / .'             .'|    .   |     \  '           |  |/  `.'   `. .   |     \  ' ' |--.\  \    .-''         '.    
           .--.. '             .'  |    |   '      |  '          |   .-.  .-.   '|   '      |  '| |    \  '  /     .-''"'-.  `.  
           |  || |            <    |    \    \     / /           |  |  |  |  |  |\    \     / / | |     |  '/     /________\   \ 
       _   |  || |             |   | ____`.   ` ..' /            |  |  |  |  |  | `.   ` ..' /  | |     |  ||                  | 
     .' |  |  |. '             |   | \ .'   '-...-'`             |  |  |  |  |  |    '-...-'`   | |     ' .'\    .-------------' 
    .   | /|  | \ '.          .|   |/  .                         |  |  |  |  |  |               | |___.' /'  \    '-.____...---. 
  .'.'| |//|__|  '. `._____.-'/|    /\  \                        |__|  |__|  |__|              /_______.'/    `.             .'  
.'.'.-'  /         `-.______ / |   |  \  \                                                     \_______|/       `''-...... -'    
.'   \_.'       _..._       `  '    \  \  \                                                                                      
             .-'_..._''.      '------'  '---'                                         _______                                    
           .' .'      '.\      .--..----.     .----.                     __.....__    \  ___ `'.                                 
          / .'                 |__| \    \   /    /                  .-''         '.   ' |--.\  \                                
         . '               .|  .--.  '   '. /'   /             .|   /     .-''"'-.  `. | |    \  '                               
    __   | |             .' |_ |  |  |    |'    /    __      .' |_ /     /________\   \| |     |  '                              
 .:--.'. | |           .'     ||  |  |    ||    | .:--.'.  .'     ||                  || |     |  |                              
/ |   \ |. '          '--.  .-'|  |  '.   `'   .'/ |   \ |'--.  .-'\    .-------------'| |     ' .'                              
`" __ | | \ '.          .|  |  |  |   \        / `" __ | |   |  |   \    '-.____...---.| |___.' /'                               
 .'.''| |  '. `._____.-'/|  |  |__|    \      /   .'.''| |   |  |    `.             .'/_______.'/                                
/ /   | |_   `-.______ / |  '.'         '----'   / /   | |_  |  '.'    `''-...... -'  \_______|/                                 
\ \._,\ '/            `  |   /                   \ \._,\ '/  |   /                                                               
 `--'  `"                `'-'                     `--'  `"   `'-'                                                                

                """
                )
            f = open("decryptionContents.txt", "w")
            self.one_level_of_decryption(file=f, sickomode=self.sickomode)

            for i in range(0, self.level):
                # open file and go through each text item
                pass

    def one_level_of_decryption(self, file=None, sickomode=None):
        for key, val in self.whatToChoose.items():
            # https://stackoverflow.com/questions/4843173/how-to-check-if-type-of-a-variable-is-string
            if not isinstance(key, str):
                key.setProbTable(val)
                ret = key.decrypt(self.text)
                if ret["IsPlaintext?"]:
                    print(ret["Plaintext"])
                    if self.cipher:
                        if ret["Extra Information"] != None:
                            print(
                                "The cipher used is",
                                ret["Cipher"] + ".",
                                ret["Extra Information"] + ".",
                            )
                        else:
                            print(ret["Cipher"])

                    return ret
        print("No encryption found. Here's the probabilities we calculated")
        import pprint

        pprint.pprint(self.whatToChoose)

def main():
    parser = argparse.ArgumentParser(
        description="Automated decryption tool. Put in the encrypted text and Ciphey will decrypt it."
    )
    # parser.add_argument('-f','--file', help='File you want to decrypt', required=False)
    # parser.add_argument('-l','--level', help='How many levels of decryption you want (the more levels, the slower it is)', required=False)
    # parser.add_argument('-g','--greppable', help='Are you grepping this output?', required=False)
    parser.add_argument("-t", "--text", help="Text to decrypt", required=False)
    # parser.add_argument('-s','--sicko-mode', help='If it is encrypted Ciphey WILL find it', required=False)
    parser.add_argument(
        "-c", "--cipher", help="What is the cipher used?", required=False
    )

    args = vars(parser.parse_args())
    if args["cipher"] != None:
        cipher = True
    else:
        cipher = False
    """
    ██████╗██╗██████╗ ██╗  ██╗███████╗██╗   ██╗
    ██╔════╝██║██╔══██╗██║  ██║██╔════╝╚██╗ ██╔╝
    ██║     ██║██████╔╝███████║█████╗   ╚████╔╝ 
    ██║     ██║██╔═══╝ ██╔══██║██╔══╝    ╚██╔╝  
    ╚██████╗██║██║     ██║  ██║███████╗   ██║ 
                Made by Brandon Skerritt"""

    # uryyb zl sngure uryyb zl zbgure naq v ernyyl qb yvxr n tbbq ratyvfu oernxsnfg
    if args["text"]:
        cipherObj = Ciphey(args["text"], cipher)
        cipherObj.decrypt()
    else:
        print(
            "You didn't supply any arguments. Look at the help menu with -h or --help"
        )
if __name__ == "__main__":
    main()


try:
    import Decryptor.basicEncryption.caesar as ca
    import Decryptor.basicEncryption.reverse as re
    import Decryptor.basicEncryption.viginere as vi
    import Decryptor.basicEncryption.pigLatin as pi
except ModuleNotFoundError:
    import app.Decryptor.basicEncryption.caesar as ca
    import app.Decryptor.basicEncryption.reverse as re
    import app.Decryptor.basicEncryption.viginere as vi
    import app.Decryptor.basicEncryption.pigLatin as pi

"""
So I want to assign the prob distribution to objects
so it makes sense to do this?
list of objects
for each item in the prob distribution
replace that with the appropriate object in the list?
So each object has a getName func that returns the name as a str

new_prob_dict = {}
for key, val in self.prob:
    for obj in list:
        if obj.getName() == key:
            new_prob_dict[obj] = val

But I don't need to do all this, do I?
The dict comes in already sorted. 
So why do I need the probability values if it's sorted?
It'd be easier if I make a list in the same order as the dict?
sooo

list_objs = [caeser, etc]
counter = 0
for key, val in self.prob:
    for listCounter, item in enumerate(list_objs):
        if item.getName() == key:
            # moves the item
            list_objs.insert(counter, list_objs.pop(listCounter))
            counter = counter + 1

Eventually we get a sorted list of obj
"""


class BasicParent:
    def __init__(self, lc):
        self.lc = lc
        self.caesar = ca.Caesar(self.lc)
        self.reverse = re.Reverse(self.lc)
        self.viginere = vi.Viginere(self.lc)
        self.pig = pi.PigLatin(self.lc)
        # self.trans = Transposition(self.lc)

        self.list_of_objects = [self.caesar, self.reverse, self.pig]

    def decrypt(self, text):
        self.text = text
        from multiprocessing.dummy import Pool as ThreadPool

        pool = ThreadPool(4)
        answers = pool.map(self.callDecrypt, self.list_of_objects)

        """for item in self.list_of_objects:
            result = item.decrypt(text)
            answers.append(result)"""
        for answer in answers:
            # adds the LC objects together
            self.lc = self.lc + answer["lc"]
            if answer["IsPlaintext?"]:
                return answer

        # so viginere runs ages
        # and you cant kill threads in a pool
        # so i just run it last lol
        result = self.callDecrypt(self.viginere)
        if result["IsPlaintext?"]:
            return result
        return {
            "lc": self.lc,
            "IsPlaintext?": False,
            "Plaintext": None,
            "Cipher": None,
            "Extra Information": None,
        }

    def callDecrypt(self, obj):
        # i only exist to call decrypt
        return obj.decrypt(self.text)

    def setProbTable(self, prob):
        """I'm still writing this"""
        self.probabilityDistribution = prob
        # we get a sorted list of objects :)
        counter = 0
        for key, val in self.probabilityDistribution.items():
            for listCounter, item in enumerate(self.list_of_objects):
                if item.getName() == key:
                    # moves the item
                    list_objs.insert(counter, list_objs.pop(listCounter))
                    counter = counter + 1
